# generated by https://github.com/puzl-cloud/kubesdk
#   filename:  apis__policy__v1_openapi.json

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime, timezone
from typing import ClassVar, Dict, List, Set

from kube_models.const import *
from kube_models.loader import *
from kube_models.loader import Loadable
from kube_models.resource import *

from ...apimachinery.pkg.apis.meta.v1 import (
    Condition,
    LabelSelector,
    ListMeta,
    ObjectMeta,
    Time,
)
from ...apimachinery.pkg.util.intstr import IntOrString


@dataclass(slots=True, kw_only=True, frozen=True)
class PodDisruptionBudgetSpec(Loadable):
    maxUnavailable: IntOrString | None = None
    minAvailable: IntOrString | None = None
    selector: LabelSelector = field(
        default_factory=dict, metadata={'x-kubernetes-patch-strategy': 'replace'}
    )
    unhealthyPodEvictionPolicy: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PodDisruptionBudgetStatus(Loadable):
    currentHealthy: int
    desiredHealthy: int
    disruptionsAllowed: int
    expectedPods: int
    conditions: List[Condition] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'type',
        },
    )
    disruptedPods: Dict[str, Time] | None = None
    observedGeneration: int | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PodDisruptionBudget(K8sResource):
    apiVersion: ClassVar[str] = 'policy/v1'
    kind: ClassVar[str] = 'PodDisruptionBudget'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    spec: PodDisruptionBudgetSpec | None = None
    status: PodDisruptionBudgetStatus | None = None
    plural_: ClassVar[str] = 'poddisruptionbudgets'
    is_namespaced_: ClassVar[bool] = True
    group_: ClassVar[Optional[str]] = 'policy'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class PodDisruptionBudgetList(Loadable):
    items: List[PodDisruptionBudget]
    apiVersion: str = 'policy/v1'
    kind: str = 'PodDisruptionBudgetList'
    metadata: ListMeta = field(default_factory=ObjectMeta)



# generated by https://github.com/puzl-cloud/kubesdk
from .v1 import BoundObjectReference, TokenRequest, TokenRequestSpec, TokenRequestStatus




__all__ = ['BoundObjectReference', 'TokenRequest', 'TokenRequestSpec', 'TokenRequestStatus']




# generated by https://github.com/puzl-cloud/kubesdk
#   filename:  apis__admissionregistration.k8s.io__v1alpha1_openapi.json

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime, timezone
from typing import ClassVar, List, Set

from kube_models.const import *
from kube_models.loader import *
from kube_models.loader import Loadable
from kube_models.resource import *

from ...apimachinery.pkg.apis.meta.v1 import LabelSelector, ListMeta, ObjectMeta


@dataclass(slots=True, kw_only=True, frozen=True)
class ApplyConfiguration(Loadable):
    expression: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class JSONPatch(Loadable):
    expression: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class MatchCondition(Loadable):
    expression: str
    name: str


@dataclass(slots=True, kw_only=True, frozen=True)
class Mutation(Loadable):
    patchType: str
    applyConfiguration: ApplyConfiguration | None = None
    jsonPatch: JSONPatch | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class NamedRuleWithOperations(Loadable):
    apiGroups: List[str] | None = None
    apiVersions: List[str] | None = None
    operations: List[str] | None = None
    resourceNames: List[str] | None = None
    resources: List[str] | None = None
    scope: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ParamKind(Loadable):
    apiVersion: str | None = None
    kind: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class Variable(Loadable):
    expression: str
    name: str


@dataclass(slots=True, kw_only=True, frozen=True)
class MatchResources(Loadable):
    excludeResourceRules: List[NamedRuleWithOperations] | None = None
    matchPolicy: str | None = None
    namespaceSelector: LabelSelector | None = None
    objectSelector: LabelSelector | None = None
    resourceRules: List[NamedRuleWithOperations] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class MutatingAdmissionPolicySpec(Loadable):
    failurePolicy: str | None = None
    matchConditions: List[MatchCondition] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'name',
        },
    )
    matchConstraints: MatchResources | None = None
    mutations: List[Mutation] | None = None
    paramKind: ParamKind | None = None
    reinvocationPolicy: str | None = None
    variables: List[Variable] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ParamRef(Loadable):
    name: str | None = None
    namespace: str | None = None
    parameterNotFoundAction: str | None = None
    selector: LabelSelector | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class MutatingAdmissionPolicy(K8sResource):
    apiVersion: ClassVar[str] = 'admissionregistration.k8s.io/v1alpha1'
    kind: ClassVar[str] = 'MutatingAdmissionPolicy'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    spec: MutatingAdmissionPolicySpec | None = None
    plural_: ClassVar[str] = 'mutatingadmissionpolicies'
    is_namespaced_: ClassVar[bool] = False
    group_: ClassVar[Optional[str]] = 'admissionregistration.k8s.io'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class MutatingAdmissionPolicyBindingSpec(Loadable):
    matchResources: MatchResources | None = None
    paramRef: ParamRef | None = None
    policyName: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class MutatingAdmissionPolicyList(Loadable):
    items: List[MutatingAdmissionPolicy]
    apiVersion: str = 'admissionregistration.k8s.io/v1alpha1'
    kind: str = 'MutatingAdmissionPolicyList'
    metadata: ListMeta = field(default_factory=ObjectMeta)


@dataclass(slots=True, kw_only=True, frozen=True)
class MutatingAdmissionPolicyBinding(K8sResource):
    apiVersion: ClassVar[str] = 'admissionregistration.k8s.io/v1alpha1'
    kind: ClassVar[str] = 'MutatingAdmissionPolicyBinding'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    spec: MutatingAdmissionPolicyBindingSpec | None = None
    plural_: ClassVar[str] = 'mutatingadmissionpolicybindings'
    is_namespaced_: ClassVar[bool] = False
    group_: ClassVar[Optional[str]] = 'admissionregistration.k8s.io'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class MutatingAdmissionPolicyBindingList(Loadable):
    items: List[MutatingAdmissionPolicyBinding]
    apiVersion: str = 'admissionregistration.k8s.io/v1alpha1'
    kind: str = 'MutatingAdmissionPolicyBindingList'
    metadata: ListMeta = field(default_factory=ObjectMeta)



# generated by https://github.com/puzl-cloud/kubesdk
from .v1 import CronJob, CronJobList, CronJobSpec, CronJobStatus, Job, JobCondition, JobList, JobSpec, JobStatus, JobTemplateSpec, PodFailurePolicy, PodFailurePolicyOnExitCodesRequirement, PodFailurePolicyOnPodConditionsPattern, PodFailurePolicyRule, SuccessPolicy, SuccessPolicyRule, UncountedTerminatedPods




__all__ = ['CronJob', 'CronJobList', 'CronJobSpec', 'CronJobStatus', 'Job', 'JobCondition', 'JobList', 'JobSpec', 'JobStatus', 'JobTemplateSpec', 'PodFailurePolicy', 'PodFailurePolicyOnExitCodesRequirement', 'PodFailurePolicyOnPodConditionsPattern', 'PodFailurePolicyRule', 'SuccessPolicy', 'SuccessPolicyRule', 'UncountedTerminatedPods']




# generated by https://github.com/puzl-cloud/kubesdk
from .v1alpha1 import ClusterTrustBundle, ClusterTrustBundleList, ClusterTrustBundleSpec, PodCertificateRequest, PodCertificateRequestList, PodCertificateRequestSpec, PodCertificateRequestStatus




__all__ = ['ClusterTrustBundle', 'ClusterTrustBundleList', 'ClusterTrustBundleSpec', 'PodCertificateRequest', 'PodCertificateRequestList', 'PodCertificateRequestSpec', 'PodCertificateRequestStatus']




# generated by https://github.com/puzl-cloud/kubesdk
#   filename:  apis__networking.k8s.io__v1_openapi.json

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime, timezone
from typing import ClassVar, List, Set

from kube_models.const import *
from kube_models.loader import *
from kube_models.loader import Loadable
from kube_models.resource import *

from ...apimachinery.pkg.apis.meta.v1 import (
    Condition,
    LabelSelector,
    ListMeta,
    ObjectMeta,
)
from ...apimachinery.pkg.util.intstr import IntOrString
from ..core.v1 import TypedLocalObjectReference


@dataclass(slots=True, kw_only=True, frozen=True)
class IPBlock(Loadable):
    cidr: str
    except_: List[str] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class IngressClassParametersReference(Loadable):
    kind: str
    name: str
    apiGroup: str | None = None
    namespace: str | None = None
    scope: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class IngressClassSpec(Loadable):
    controller: str | None = None
    parameters: IngressClassParametersReference | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class IngressPortStatus(Loadable):
    port: int
    protocol: str
    error: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class IngressTLS(Loadable):
    hosts: List[str] | None = None
    secretName: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ParentReference(Loadable):
    name: str
    resource: str
    group: str | None = None
    namespace: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ServiceBackendPort(Loadable):
    name: str | None = None
    number: int | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ServiceCIDRSpec(Loadable):
    cidrs: List[str] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class IPAddressSpec(Loadable):
    parentRef: ParentReference


@dataclass(slots=True, kw_only=True, frozen=True)
class IngressLoadBalancerIngress(Loadable):
    hostname: str | None = None
    ip: str | None = None
    ports: List[IngressPortStatus] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class IngressLoadBalancerStatus(Loadable):
    ingress: List[IngressLoadBalancerIngress] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class IngressServiceBackend(Loadable):
    name: str
    port: ServiceBackendPort | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class IngressStatus(Loadable):
    loadBalancer: IngressLoadBalancerStatus | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class NetworkPolicyPort(Loadable):
    endPort: int | None = None
    port: IntOrString | None = None
    protocol: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class IPAddress(K8sResource):
    apiVersion: ClassVar[str] = 'networking.k8s.io/v1'
    kind: ClassVar[str] = 'IPAddress'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    spec: IPAddressSpec | None = None
    plural_: ClassVar[str] = 'ipaddresses'
    is_namespaced_: ClassVar[bool] = False
    group_: ClassVar[Optional[str]] = 'networking.k8s.io'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class IPAddressList(Loadable):
    items: List[IPAddress]
    apiVersion: str = 'networking.k8s.io/v1'
    kind: str = 'IPAddressList'
    metadata: ListMeta = field(default_factory=ObjectMeta)


@dataclass(slots=True, kw_only=True, frozen=True)
class IngressBackend(Loadable):
    resource: TypedLocalObjectReference | None = None
    service: IngressServiceBackend | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class IngressClass(K8sResource):
    apiVersion: ClassVar[str] = 'networking.k8s.io/v1'
    kind: ClassVar[str] = 'IngressClass'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    spec: IngressClassSpec | None = None
    plural_: ClassVar[str] = 'ingressclasses'
    is_namespaced_: ClassVar[bool] = False
    group_: ClassVar[Optional[str]] = 'networking.k8s.io'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class IngressClassList(Loadable):
    items: List[IngressClass]
    apiVersion: str = 'networking.k8s.io/v1'
    kind: str = 'IngressClassList'
    metadata: ListMeta = field(default_factory=ObjectMeta)


@dataclass(slots=True, kw_only=True, frozen=True)
class NetworkPolicyPeer(Loadable):
    ipBlock: IPBlock | None = None
    namespaceSelector: LabelSelector | None = None
    podSelector: LabelSelector | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ServiceCIDRStatus(Loadable):
    conditions: List[Condition] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'type',
        },
    )


@dataclass(slots=True, kw_only=True, frozen=True)
class HTTPIngressPath(Loadable):
    backend: IngressBackend
    pathType: str
    path: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class HTTPIngressRuleValue(Loadable):
    paths: List[HTTPIngressPath]


@dataclass(slots=True, kw_only=True, frozen=True)
class IngressRule(Loadable):
    host: str | None = None
    http: HTTPIngressRuleValue | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class IngressSpec(Loadable):
    defaultBackend: IngressBackend | None = None
    ingressClassName: str | None = None
    rules: List[IngressRule] | None = None
    tls: List[IngressTLS] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class NetworkPolicyEgressRule(Loadable):
    ports: List[NetworkPolicyPort] | None = None
    to: List[NetworkPolicyPeer] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class NetworkPolicyIngressRule(Loadable):
    from_: List[NetworkPolicyPeer] | None = None
    ports: List[NetworkPolicyPort] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class NetworkPolicySpec(Loadable):
    egress: List[NetworkPolicyEgressRule] | None = None
    ingress: List[NetworkPolicyIngressRule] | None = None
    podSelector: LabelSelector | None = None
    policyTypes: List[str] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ServiceCIDR(K8sResource):
    apiVersion: ClassVar[str] = 'networking.k8s.io/v1'
    kind: ClassVar[str] = 'ServiceCIDR'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    spec: ServiceCIDRSpec | None = None
    status: ServiceCIDRStatus | None = None
    plural_: ClassVar[str] = 'servicecidrs'
    is_namespaced_: ClassVar[bool] = False
    group_: ClassVar[Optional[str]] = 'networking.k8s.io'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class ServiceCIDRList(Loadable):
    items: List[ServiceCIDR]
    apiVersion: str = 'networking.k8s.io/v1'
    kind: str = 'ServiceCIDRList'
    metadata: ListMeta = field(default_factory=ObjectMeta)


@dataclass(slots=True, kw_only=True, frozen=True)
class Ingress(K8sResource):
    apiVersion: ClassVar[str] = 'networking.k8s.io/v1'
    kind: ClassVar[str] = 'Ingress'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    spec: IngressSpec | None = None
    status: IngressStatus | None = None
    plural_: ClassVar[str] = 'ingresses'
    is_namespaced_: ClassVar[bool] = True
    group_: ClassVar[Optional[str]] = 'networking.k8s.io'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class IngressList(Loadable):
    items: List[Ingress]
    apiVersion: str = 'networking.k8s.io/v1'
    kind: str = 'IngressList'
    metadata: ListMeta = field(default_factory=ObjectMeta)


@dataclass(slots=True, kw_only=True, frozen=True)
class NetworkPolicy(K8sResource):
    apiVersion: ClassVar[str] = 'networking.k8s.io/v1'
    kind: ClassVar[str] = 'NetworkPolicy'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    spec: NetworkPolicySpec | None = None
    plural_: ClassVar[str] = 'networkpolicies'
    is_namespaced_: ClassVar[bool] = True
    group_: ClassVar[Optional[str]] = 'networking.k8s.io'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class NetworkPolicyList(Loadable):
    items: List[NetworkPolicy]
    apiVersion: str = 'networking.k8s.io/v1'
    kind: str = 'NetworkPolicyList'
    metadata: ListMeta = field(default_factory=ObjectMeta)



# generated by https://github.com/puzl-cloud/kubesdk
from .v1 import HTTPIngressPath, HTTPIngressRuleValue, IPAddress, IPAddressList, IPAddressSpec, IPBlock, Ingress, IngressBackend, IngressClass, IngressClassList, IngressClassParametersReference, IngressClassSpec, IngressList, IngressLoadBalancerIngress, IngressLoadBalancerStatus, IngressPortStatus, IngressRule, IngressServiceBackend, IngressSpec, IngressStatus, IngressTLS, NetworkPolicy, NetworkPolicyEgressRule, NetworkPolicyIngressRule, NetworkPolicyList, NetworkPolicyPeer, NetworkPolicyPort, NetworkPolicySpec, ParentReference, ServiceBackendPort, ServiceCIDR, ServiceCIDRList, ServiceCIDRSpec, ServiceCIDRStatus




__all__ = ['HTTPIngressPath', 'HTTPIngressRuleValue', 'IPAddress', 'IPAddressList', 'IPAddressSpec', 'IPBlock', 'Ingress', 'IngressBackend', 'IngressClass', 'IngressClassList', 'IngressClassParametersReference', 'IngressClassSpec', 'IngressList', 'IngressLoadBalancerIngress', 'IngressLoadBalancerStatus', 'IngressPortStatus', 'IngressRule', 'IngressServiceBackend', 'IngressSpec', 'IngressStatus', 'IngressTLS', 'NetworkPolicy', 'NetworkPolicyEgressRule', 'NetworkPolicyIngressRule', 'NetworkPolicyList', 'NetworkPolicyPeer', 'NetworkPolicyPort', 'NetworkPolicySpec', 'ParentReference', 'ServiceBackendPort', 'ServiceCIDR', 'ServiceCIDRList', 'ServiceCIDRSpec', 'ServiceCIDRStatus']




# generated by https://github.com/puzl-cloud/kubesdk
#   filename:  apis__authorization.k8s.io_openapi.json

from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime, timezone
from typing import ClassVar, List, Set

from kube_models.const import *
from kube_models.loader import *
from kube_models.loader import Loadable
from kube_models.resource import *


@dataclass(slots=True, kw_only=True, frozen=True)
class GroupVersionForDiscovery(Loadable):
    groupVersion: str
    version: str


@dataclass(slots=True, kw_only=True, frozen=True)
class ServerAddressByClientCIDR(Loadable):
    clientCIDR: str
    serverAddress: str


@dataclass(slots=True, kw_only=True, frozen=True)
class APIGroup(Loadable):
    name: str
    versions: List[GroupVersionForDiscovery]
    apiVersion: str = 'v1'
    kind: str = 'APIGroup'
    preferredVersion: GroupVersionForDiscovery | None = None
    serverAddressByClientCIDRs: List[ServerAddressByClientCIDR] | None = None



import sys

if sys.version_info >= (3, 13):
    from ._resource_list_pep695 import K8sResourceList, K8sResource
else:
    from ._resource_list_generics import K8sResourceList, K8sResource


__all__ = ["K8sResourceList", "K8sResource"]



from typing import List, ClassVar
from dataclasses import dataclass, field
from weakref import WeakKeyDictionary
from threading import RLock

# noinspection ALL
from ._k8s_resource_base import K8sResource, ListMeta, _bind_class_vars_from_original_kind


@dataclass(slots=True, kw_only=True, frozen=True)
class K8sResourceList[ResourceT: K8sResource](K8sResource):
    items: List[ResourceT]
    metadata: ListMeta = field(default_factory=ListMeta)

    _type_cache: ClassVar[WeakKeyDictionary] = WeakKeyDictionary()
    _type_cache_lock: ClassVar[RLock] = RLock()

    # Bind apiVersion to the List from the original kind dynamically
    def __class_getitem__(cls, params): return _bind_class_vars_from_original_kind(cls, params)



# generated by https://github.com/puzl-cloud/kubesdk
from .v1 import NodeSelector, NodeSelectorRequirement, NodeSelectorTerm




__all__ = ['NodeSelector', 'NodeSelectorRequirement', 'NodeSelectorTerm']




# generated by https://github.com/puzl-cloud/kubesdk
#   filename:  apis__certificates.k8s.io__v1_openapi.json

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime, timezone
from typing import ClassVar, Dict, List, Set

from kube_models.const import *
from kube_models.loader import *
from kube_models.loader import Loadable
from kube_models.resource import *

from ...apimachinery.pkg.apis.meta.v1 import ListMeta, ObjectMeta, Time


@dataclass(slots=True, kw_only=True, frozen=True)
class CertificateSigningRequestSpec(Loadable):
    request: str
    signerName: str
    expirationSeconds: int | None = None
    extra: Dict[str, List[str]] | None = None
    groups: List[str] | None = None
    uid: str | None = None
    usages: List[str] | None = None
    username: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class CertificateSigningRequestCondition(Loadable):
    status: str
    type: str
    lastTransitionTime: Time | None = None
    lastUpdateTime: Time | None = None
    message: str | None = None
    reason: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class CertificateSigningRequestStatus(Loadable):
    certificate: str | None = None
    conditions: List[CertificateSigningRequestCondition] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class CertificateSigningRequest(K8sResource):
    spec: CertificateSigningRequestSpec
    apiVersion: ClassVar[str] = 'certificates.k8s.io/v1'
    kind: ClassVar[str] = 'CertificateSigningRequest'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    status: CertificateSigningRequestStatus | None = None
    plural_: ClassVar[str] = 'certificatesigningrequests'
    is_namespaced_: ClassVar[bool] = False
    group_: ClassVar[Optional[str]] = 'certificates.k8s.io'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class CertificateSigningRequestList(Loadable):
    items: List[CertificateSigningRequest]
    apiVersion: str = 'certificates.k8s.io/v1'
    kind: str = 'CertificateSigningRequestList'
    metadata: ListMeta = field(default_factory=ObjectMeta)



# generated by https://github.com/puzl-cloud/kubesdk
from .v1 import FieldSelectorAttributes, LabelSelectorAttributes, LocalSubjectAccessReview, NonResourceAttributes, NonResourceRule, ResourceAttributes, ResourceRule, SelfSubjectAccessReview, SelfSubjectAccessReviewSpec, SelfSubjectRulesReview, SelfSubjectRulesReviewSpec, SubjectAccessReview, SubjectAccessReviewSpec, SubjectAccessReviewStatus, SubjectRulesReviewStatus




__all__ = ['FieldSelectorAttributes', 'LabelSelectorAttributes', 'LocalSubjectAccessReview', 'NonResourceAttributes', 'NonResourceRule', 'ResourceAttributes', 'ResourceRule', 'SelfSubjectAccessReview', 'SelfSubjectAccessReviewSpec', 'SelfSubjectRulesReview', 'SelfSubjectRulesReviewSpec', 'SubjectAccessReview', 'SubjectAccessReviewSpec', 'SubjectAccessReviewStatus', 'SubjectRulesReviewStatus']




# generated by https://github.com/puzl-cloud/kubesdk
from .v1 import APIResource, APIResourceList, Condition, DeleteOptions, FieldsV1, LabelSelector, LabelSelectorRequirement, ListMeta, ManagedFieldsEntry, MicroTime, ObjectMeta, OwnerReference, Patch, Preconditions, Status, StatusCause, StatusDetails, Time, WatchEvent




__all__ = ['APIResource', 'APIResourceList', 'Condition', 'DeleteOptions', 'FieldsV1', 'LabelSelector', 'LabelSelectorRequirement', 'ListMeta', 'ManagedFieldsEntry', 'MicroTime', 'ObjectMeta', 'OwnerReference', 'Patch', 'Preconditions', 'Status', 'StatusCause', 'StatusDetails', 'Time', 'WatchEvent']




# generated by https://github.com/puzl-cloud/kubesdk
from .v1 import APIResource, APIResourceList, FieldSelectorRequirement, FieldsV1, LabelSelectorRequirement, ManagedFieldsEntry, ObjectMeta, OwnerReference, Time




__all__ = ['APIResource', 'APIResourceList', 'FieldSelectorRequirement', 'FieldsV1', 'LabelSelectorRequirement', 'ManagedFieldsEntry', 'ObjectMeta', 'OwnerReference', 'Time']




from __future__ import annotations

import unittest
import pkgutil
import importlib
from dataclasses import dataclass, field

import kube_models
from kube_models.registry import ALL_RESOURCES, register_model, maybe_get_model_key, get_model_by_body, get_model


class TestRegistry(unittest.TestCase):
    def setUp(self) -> None:
        ALL_RESOURCES.clear()

    def test_key_from_class_attributes_and_basic_lookup(self) -> None:
        class ModelWithAttributes:
            apiVersion = "v1"
            kind = "Secret"

        self.assertEqual(("v1", "Secret"), maybe_get_model_key(ModelWithAttributes))

        register_model(ModelWithAttributes)
        self.assertIs(ModelWithAttributes, get_model("v1", "Secret"))
        self.assertIsNone(get_model("v1", "DoesNotExist"))

    def test_key_from_slots_dataclass_default_values(self) -> None:
        @dataclass(slots=True, frozen=True)
        class SlotsDataclassDefaults:
            apiVersion: str = "apps/v1"
            kind: str = "Deployment"
        self.assertEqual(("apps/v1", "Deployment"), maybe_get_model_key(SlotsDataclassDefaults))
        register_model(SlotsDataclassDefaults)
        self.assertIs(SlotsDataclassDefaults, get_model("apps/v1", "Deployment"))

    def test_key_from_slots_dataclass_default_factory(self) -> None:
        @dataclass(slots=True, frozen=True)
        class SlotsDataclassFactories:
            apiVersion: str = field(default_factory=lambda: "batch/v1")
            kind: str = field(default_factory=lambda: "Job")

        self.assertEqual(("batch/v1", "Job"), maybe_get_model_key(SlotsDataclassFactories))

    def test_dataclass_fields_missing_one_side(self) -> None:
        @dataclass(slots=True, frozen=True)
        class OnlyKind:
            kind: str = "Pod"

        # apiVersion field is missing -> covers field_object is None branch
        self.assertIsNone(maybe_get_model_key(OnlyKind))

    def test_dataclass_fields_present_but_not_dict(self) -> None:
        class NotADataclass:
            __dataclass_fields__ = "not-a-dict"

        self.assertIsNone(maybe_get_model_key(NotADataclass))

    def test_dataclass_fields_no_string_defaults(self) -> None:
        @dataclass(slots=True, frozen=True)
        class NoDefaults:
            apiVersion: str
            kind: str

        self.assertIsNone(maybe_get_model_key(NoDefaults))

        @dataclass(slots=True, frozen=True)
        class NonStringDefaults:
            apiVersion: int = 1  # not a string
            kind: str = "Thing"

        self.assertIsNone(maybe_get_model_key(NonStringDefaults))

    def test_register_model_does_not_override_existing(self) -> None:
        class FirstRegistered:
            apiVersion = "v1"
            kind = "ConfigMap"

        class SecondRegistered:
            apiVersion = "v1"
            kind = "ConfigMap"

        register_model(FirstRegistered)
        register_model(SecondRegistered)
        self.assertIs(FirstRegistered, get_model("v1", "ConfigMap"))

    def test_get_model_by_body_validation(self) -> None:
        class BodyModel:
            apiVersion = "v1"
            kind = "Service"

        register_model(BodyModel)

        self.assertIsNone(get_model_by_body({}))
        self.assertIsNone(get_model_by_body({"apiVersion": "v1", "kind": None}))
        self.assertIsNone(get_model_by_body({"apiVersion": None, "kind": "Service"}))

        self.assertIs(BodyModel, get_model_by_body({"apiVersion": "v1", "kind": "Service"}))

    def test_import_all(self):
        initial_len = len(ALL_RESOURCES)
        pkg = kube_models
        prefix = pkg.__name__ + "."
        for _finder, modname, _ispkg in pkgutil.walk_packages(pkg.__path__, prefix=prefix):
            # Skip private files/modules
            if modname.rsplit(".", 1)[-1].startswith("_"):
                continue

            importlib.import_module(modname)

        final_len = len(ALL_RESOURCES)
        self.assertGreater(final_len, initial_len)



# generated by https://github.com/puzl-cloud/kubesdk
from .v1 import SelfSubjectReview, SelfSubjectReviewStatus, TokenReview, TokenReviewSpec, TokenReviewStatus, UserInfo




__all__ = ['SelfSubjectReview', 'SelfSubjectReviewStatus', 'TokenReview', 'TokenReviewSpec', 'TokenReviewStatus', 'UserInfo']




# generated by https://github.com/puzl-cloud/kubesdk
#   filename:  apis__apps__v1_openapi.json

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime, timezone
from typing import ClassVar, Dict, List, Set

from kube_models.const import *
from kube_models.loader import *
from kube_models.loader import Loadable
from kube_models.resource import *

from ...apimachinery.pkg.api.resource import Quantity
from ...apimachinery.pkg.apis.meta.v1 import LabelSelector, ObjectMeta, Time
from ...apimachinery.pkg.util.intstr import IntOrString


@dataclass(slots=True, kw_only=True, frozen=True)
class AWSElasticBlockStoreVolumeSource(Loadable):
    volumeID: str
    fsType: str | None = None
    partition: int | None = None
    readOnly: bool | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class AppArmorProfile(Loadable):
    type: str
    localhostProfile: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class AzureDiskVolumeSource(Loadable):
    diskName: str
    diskURI: str
    cachingMode: str | None = 'ReadWrite'
    fsType: str | None = 'ext4'
    kind: str | None = 'Shared'
    readOnly: bool | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class AzureFileVolumeSource(Loadable):
    secretName: str
    shareName: str
    readOnly: bool | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class Capabilities(Loadable):
    add: List[str] | None = None
    drop: List[str] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ConfigMapEnvSource(Loadable):
    name: str | None = None
    optional: bool | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ConfigMapKeySelector(Loadable):
    key: str
    name: str | None = None
    optional: bool | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ContainerPort(Loadable):
    containerPort: int
    hostIP: str | None = None
    hostPort: int | None = None
    name: str | None = None
    protocol: str | None = 'TCP'


@dataclass(slots=True, kw_only=True, frozen=True)
class ContainerResizePolicy(Loadable):
    resourceName: str
    restartPolicy: str


@dataclass(slots=True, kw_only=True, frozen=True)
class ContainerRestartRuleOnExitCodes(Loadable):
    operator: str
    values: List[int] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ExecAction(Loadable):
    command: List[str] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class FCVolumeSource(Loadable):
    fsType: str | None = None
    lun: int | None = None
    readOnly: bool | None = None
    targetWWNs: List[str] | None = None
    wwids: List[str] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class FileKeySelector(Loadable):
    key: str
    path: str
    volumeName: str
    optional: bool | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class FlockerVolumeSource(Loadable):
    datasetName: str | None = None
    datasetUUID: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class GCEPersistentDiskVolumeSource(Loadable):
    pdName: str
    fsType: str | None = None
    partition: int | None = None
    readOnly: bool | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class GRPCAction(Loadable):
    port: int
    service: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class GitRepoVolumeSource(Loadable):
    repository: str
    directory: str | None = None
    revision: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class GlusterfsVolumeSource(Loadable):
    endpoints: str
    path: str
    readOnly: bool | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class HTTPHeader(Loadable):
    name: str
    value: str


@dataclass(slots=True, kw_only=True, frozen=True)
class HostAlias(Loadable):
    ip: str
    hostnames: List[str] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class HostPathVolumeSource(Loadable):
    path: str
    type: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ImageVolumeSource(Loadable):
    pullPolicy: str | None = None
    reference: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class KeyToPath(Loadable):
    key: str
    path: str
    mode: int | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class LocalObjectReference(Loadable):
    name: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ModifyVolumeStatus(Loadable):
    status: str
    targetVolumeAttributesClassName: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class NFSVolumeSource(Loadable):
    path: str
    server: str
    readOnly: bool | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class NodeSelectorRequirement(Loadable):
    key: str
    operator: str
    values: List[str] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class NodeSelectorTerm(Loadable):
    matchExpressions: List[NodeSelectorRequirement] | None = None
    matchFields: List[NodeSelectorRequirement] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ObjectFieldSelector(Loadable):
    fieldPath: str
    apiVersion: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PersistentVolumeClaimVolumeSource(Loadable):
    claimName: str
    readOnly: bool | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PhotonPersistentDiskVolumeSource(Loadable):
    pdID: str
    fsType: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PodCertificateProjection(Loadable):
    keyType: str
    signerName: str
    certificateChainPath: str | None = None
    credentialBundlePath: str | None = None
    keyPath: str | None = None
    maxExpirationSeconds: int | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PodDNSConfigOption(Loadable):
    name: str | None = None
    value: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PodOS(Loadable):
    name: str


@dataclass(slots=True, kw_only=True, frozen=True)
class PodReadinessGate(Loadable):
    conditionType: str


@dataclass(slots=True, kw_only=True, frozen=True)
class PodResourceClaim(Loadable):
    name: str
    resourceClaimName: str | None = None
    resourceClaimTemplateName: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PodSchedulingGate(Loadable):
    name: str


@dataclass(slots=True, kw_only=True, frozen=True)
class PortworxVolumeSource(Loadable):
    volumeID: str
    fsType: str | None = None
    readOnly: bool | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PreferredSchedulingTerm(Loadable):
    preference: NodeSelectorTerm
    weight: int


@dataclass(slots=True, kw_only=True, frozen=True)
class QuobyteVolumeSource(Loadable):
    registry: str
    volume: str
    group: str | None = None
    readOnly: bool | None = None
    tenant: str | None = None
    user: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class RBDVolumeSource(Loadable):
    image: str
    monitors: List[str]
    fsType: str | None = None
    keyring: str | None = '/etc/ceph/keyring'
    pool: str | None = 'rbd'
    readOnly: bool | None = None
    secretRef: LocalObjectReference | None = None
    user: str | None = 'admin'


@dataclass(slots=True, kw_only=True, frozen=True)
class ResourceClaim(Loadable):
    name: str
    request: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class SELinuxOptions(Loadable):
    level: str | None = None
    role: str | None = None
    type: str | None = None
    user: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ScaleIOVolumeSource(Loadable):
    gateway: str
    secretRef: LocalObjectReference
    system: str
    fsType: str | None = 'xfs'
    protectionDomain: str | None = None
    readOnly: bool | None = None
    sslEnabled: bool | None = None
    storageMode: str | None = 'ThinProvisioned'
    storagePool: str | None = None
    volumeName: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class SeccompProfile(Loadable):
    type: str
    localhostProfile: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class SecretEnvSource(Loadable):
    name: str | None = None
    optional: bool | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class SecretKeySelector(Loadable):
    key: str
    name: str | None = None
    optional: bool | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class SecretProjection(Loadable):
    items: List[KeyToPath] | None = None
    name: str | None = None
    optional: bool | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class SecretVolumeSource(Loadable):
    defaultMode: int | None = None
    items: List[KeyToPath] | None = None
    optional: bool | None = None
    secretName: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ServiceAccountTokenProjection(Loadable):
    path: str
    audience: str | None = None
    expirationSeconds: int | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class SleepAction(Loadable):
    seconds: int


@dataclass(slots=True, kw_only=True, frozen=True)
class StorageOSVolumeSource(Loadable):
    fsType: str | None = None
    readOnly: bool | None = None
    secretRef: LocalObjectReference | None = None
    volumeName: str | None = None
    volumeNamespace: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class Sysctl(Loadable):
    name: str
    value: str


@dataclass(slots=True, kw_only=True, frozen=True)
class Toleration(Loadable):
    effect: str | None = None
    key: str | None = None
    operator: str | None = None
    tolerationSeconds: int | None = None
    value: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class TypedLocalObjectReference(Loadable):
    kind: str
    name: str
    apiGroup: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class TypedObjectReference(Loadable):
    kind: str
    name: str
    apiGroup: str | None = None
    namespace: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class VolumeDevice(Loadable):
    devicePath: str
    name: str


@dataclass(slots=True, kw_only=True, frozen=True)
class VolumeMount(Loadable):
    mountPath: str
    name: str
    mountPropagation: str | None = None
    readOnly: bool | None = None
    recursiveReadOnly: str | None = None
    subPath: str | None = None
    subPathExpr: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class VsphereVirtualDiskVolumeSource(Loadable):
    volumePath: str
    fsType: str | None = None
    storagePolicyID: str | None = None
    storagePolicyName: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class WindowsSecurityContextOptions(Loadable):
    gmsaCredentialSpec: str | None = None
    gmsaCredentialSpecName: str | None = None
    hostProcess: bool | None = None
    runAsUserName: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class CSIVolumeSource(Loadable):
    driver: str
    fsType: str | None = None
    nodePublishSecretRef: LocalObjectReference | None = None
    readOnly: bool | None = None
    volumeAttributes: Dict[str, str] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class CephFSVolumeSource(Loadable):
    monitors: List[str]
    path: str | None = None
    readOnly: bool | None = None
    secretFile: str | None = None
    secretRef: LocalObjectReference | None = None
    user: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class CinderVolumeSource(Loadable):
    volumeID: str
    fsType: str | None = None
    readOnly: bool | None = None
    secretRef: LocalObjectReference | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ConfigMapProjection(Loadable):
    items: List[KeyToPath] | None = None
    name: str | None = None
    optional: bool | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ConfigMapVolumeSource(Loadable):
    defaultMode: int | None = None
    items: List[KeyToPath] | None = None
    name: str | None = None
    optional: bool | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ContainerRestartRule(Loadable):
    action: str
    exitCodes: ContainerRestartRuleOnExitCodes | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class EmptyDirVolumeSource(Loadable):
    medium: str | None = None
    sizeLimit: Quantity | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class EnvFromSource(Loadable):
    configMapRef: ConfigMapEnvSource | None = None
    prefix: str | None = None
    secretRef: SecretEnvSource | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class FlexVolumeSource(Loadable):
    driver: str
    fsType: str | None = None
    options: Dict[str, str] | None = None
    readOnly: bool | None = None
    secretRef: LocalObjectReference | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class HTTPGetAction(Loadable):
    port: IntOrString
    host: str | None = None
    httpHeaders: List[HTTPHeader] | None = None
    path: str | None = None
    scheme: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ISCSIVolumeSource(Loadable):
    iqn: str
    lun: int
    targetPortal: str
    chapAuthDiscovery: bool | None = None
    chapAuthSession: bool | None = None
    fsType: str | None = None
    initiatorName: str | None = None
    iscsiInterface: str | None = 'default'
    portals: List[str] | None = None
    readOnly: bool | None = None
    secretRef: LocalObjectReference | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class NodeSelector(Loadable):
    nodeSelectorTerms: List[NodeSelectorTerm]


@dataclass(slots=True, kw_only=True, frozen=True)
class PersistentVolumeClaimCondition(Loadable):
    status: str
    type: str
    lastProbeTime: Time | None = None
    lastTransitionTime: Time | None = None
    message: str | None = None
    reason: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PersistentVolumeClaimStatus(Loadable):
    accessModes: List[str] | None = None
    allocatedResourceStatuses: Dict[str, str] | None = None
    allocatedResources: Dict[str, Quantity] | None = None
    capacity: Dict[str, Quantity] | None = None
    conditions: List[PersistentVolumeClaimCondition] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'type',
        },
    )
    currentVolumeAttributesClassName: str | None = None
    modifyVolumeStatus: ModifyVolumeStatus | None = None
    phase: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PodDNSConfig(Loadable):
    nameservers: List[str] | None = None
    options: List[PodDNSConfigOption] | None = None
    searches: List[str] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PodSecurityContext(Loadable):
    appArmorProfile: AppArmorProfile | None = None
    fsGroup: int | None = None
    fsGroupChangePolicy: str | None = None
    runAsGroup: int | None = None
    runAsNonRoot: bool | None = None
    runAsUser: int | None = None
    seLinuxChangePolicy: str | None = None
    seLinuxOptions: SELinuxOptions | None = None
    seccompProfile: SeccompProfile | None = None
    supplementalGroups: List[int] | None = None
    supplementalGroupsPolicy: str | None = None
    sysctls: List[Sysctl] | None = None
    windowsOptions: WindowsSecurityContextOptions | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ResourceFieldSelector(Loadable):
    resource: str
    containerName: str | None = None
    divisor: Quantity | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ResourceRequirements(Loadable):
    claims: List[ResourceClaim] | None = None
    limits: Dict[str, Quantity] | None = None
    requests: Dict[str, Quantity] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class SecurityContext(Loadable):
    allowPrivilegeEscalation: bool | None = None
    appArmorProfile: AppArmorProfile | None = None
    capabilities: Capabilities | None = None
    privileged: bool | None = None
    procMount: str | None = None
    readOnlyRootFilesystem: bool | None = None
    runAsGroup: int | None = None
    runAsNonRoot: bool | None = None
    runAsUser: int | None = None
    seLinuxOptions: SELinuxOptions | None = None
    seccompProfile: SeccompProfile | None = None
    windowsOptions: WindowsSecurityContextOptions | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class TCPSocketAction(Loadable):
    port: IntOrString
    host: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class VolumeResourceRequirements(Loadable):
    limits: Dict[str, Quantity] | None = None
    requests: Dict[str, Quantity] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ClusterTrustBundleProjection(Loadable):
    path: str
    labelSelector: LabelSelector | None = None
    name: str | None = None
    optional: bool | None = None
    signerName: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class DownwardAPIVolumeFile(Loadable):
    path: str
    fieldRef: ObjectFieldSelector | None = None
    mode: int | None = None
    resourceFieldRef: ResourceFieldSelector | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class DownwardAPIVolumeSource(Loadable):
    defaultMode: int | None = None
    items: List[DownwardAPIVolumeFile] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class EnvVarSource(Loadable):
    configMapKeyRef: ConfigMapKeySelector | None = None
    fieldRef: ObjectFieldSelector | None = None
    fileKeyRef: FileKeySelector | None = None
    resourceFieldRef: ResourceFieldSelector | None = None
    secretKeyRef: SecretKeySelector | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class LifecycleHandler(Loadable):
    exec: ExecAction | None = None
    httpGet: HTTPGetAction | None = None
    sleep: SleepAction | None = None
    tcpSocket: TCPSocketAction | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class NodeAffinity(Loadable):
    preferredDuringSchedulingIgnoredDuringExecution: (
        List[PreferredSchedulingTerm] | None
    ) = None
    requiredDuringSchedulingIgnoredDuringExecution: NodeSelector | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PersistentVolumeClaimSpec(Loadable):
    accessModes: List[str] | None = None
    dataSource: TypedLocalObjectReference | None = None
    dataSourceRef: TypedObjectReference | None = None
    resources: VolumeResourceRequirements | None = None
    selector: LabelSelector | None = None
    storageClassName: str | None = None
    volumeAttributesClassName: str | None = None
    volumeMode: str | None = None
    volumeName: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PersistentVolumeClaimTemplate(Loadable):
    spec: PersistentVolumeClaimSpec
    metadata: ObjectMeta | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PodAffinityTerm(Loadable):
    topologyKey: str
    labelSelector: LabelSelector | None = None
    matchLabelKeys: List[str] | None = None
    mismatchLabelKeys: List[str] | None = None
    namespaceSelector: LabelSelector | None = None
    namespaces: List[str] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class Probe(Loadable):
    exec: ExecAction | None = None
    failureThreshold: int | None = None
    grpc: GRPCAction | None = None
    httpGet: HTTPGetAction | None = None
    initialDelaySeconds: int | None = None
    periodSeconds: int | None = None
    successThreshold: int | None = None
    tcpSocket: TCPSocketAction | None = None
    terminationGracePeriodSeconds: int | None = None
    timeoutSeconds: int | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class TopologySpreadConstraint(Loadable):
    maxSkew: int
    topologyKey: str
    whenUnsatisfiable: str
    labelSelector: LabelSelector | None = None
    matchLabelKeys: List[str] | None = None
    minDomains: int | None = None
    nodeAffinityPolicy: str | None = None
    nodeTaintsPolicy: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class WeightedPodAffinityTerm(Loadable):
    podAffinityTerm: PodAffinityTerm
    weight: int


@dataclass(slots=True, kw_only=True, frozen=True)
class DownwardAPIProjection(Loadable):
    items: List[DownwardAPIVolumeFile] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class EnvVar(Loadable):
    name: str
    value: str | None = None
    valueFrom: EnvVarSource | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class EphemeralVolumeSource(Loadable):
    volumeClaimTemplate: PersistentVolumeClaimTemplate | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class Lifecycle(Loadable):
    postStart: LifecycleHandler | None = None
    preStop: LifecycleHandler | None = None
    stopSignal: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PersistentVolumeClaim(Loadable):
    apiVersion: str = 'v1'
    kind: str = 'PersistentVolumeClaim'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    spec: PersistentVolumeClaimSpec | None = None
    status: PersistentVolumeClaimStatus | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PodAffinity(Loadable):
    preferredDuringSchedulingIgnoredDuringExecution: (
        List[WeightedPodAffinityTerm] | None
    ) = None
    requiredDuringSchedulingIgnoredDuringExecution: List[PodAffinityTerm] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PodAntiAffinity(Loadable):
    preferredDuringSchedulingIgnoredDuringExecution: (
        List[WeightedPodAffinityTerm] | None
    ) = None
    requiredDuringSchedulingIgnoredDuringExecution: List[PodAffinityTerm] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class VolumeProjection(Loadable):
    clusterTrustBundle: ClusterTrustBundleProjection | None = None
    configMap: ConfigMapProjection | None = None
    downwardAPI: DownwardAPIProjection | None = None
    podCertificate: PodCertificateProjection | None = None
    secret: SecretProjection | None = None
    serviceAccountToken: ServiceAccountTokenProjection | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class Affinity(Loadable):
    nodeAffinity: NodeAffinity | None = None
    podAffinity: PodAffinity | None = None
    podAntiAffinity: PodAntiAffinity | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class Container(Loadable):
    name: str
    args: List[str] | None = None
    command: List[str] | None = None
    env: List[EnvVar] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'name',
        },
    )
    envFrom: List[EnvFromSource] | None = None
    image: str | None = None
    imagePullPolicy: str | None = None
    lifecycle: Lifecycle | None = None
    livenessProbe: Probe | None = None
    ports: List[ContainerPort] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'containerPort',
        },
    )
    readinessProbe: Probe | None = None
    resizePolicy: List[ContainerResizePolicy] | None = None
    resources: ResourceRequirements | None = None
    restartPolicy: str | None = None
    restartPolicyRules: List[ContainerRestartRule] | None = None
    securityContext: SecurityContext | None = None
    startupProbe: Probe | None = None
    stdin: bool | None = None
    stdinOnce: bool | None = None
    terminationMessagePath: str | None = None
    terminationMessagePolicy: str | None = None
    tty: bool | None = None
    volumeDevices: List[VolumeDevice] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'devicePath',
        },
    )
    volumeMounts: List[VolumeMount] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'mountPath',
        },
    )
    workingDir: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class EphemeralContainer(Loadable):
    name: str
    args: List[str] | None = None
    command: List[str] | None = None
    env: List[EnvVar] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'name',
        },
    )
    envFrom: List[EnvFromSource] | None = None
    image: str | None = None
    imagePullPolicy: str | None = None
    lifecycle: Lifecycle | None = None
    livenessProbe: Probe | None = None
    ports: List[ContainerPort] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'containerPort',
        },
    )
    readinessProbe: Probe | None = None
    resizePolicy: List[ContainerResizePolicy] | None = None
    resources: ResourceRequirements | None = None
    restartPolicy: str | None = None
    restartPolicyRules: List[ContainerRestartRule] | None = None
    securityContext: SecurityContext | None = None
    startupProbe: Probe | None = None
    stdin: bool | None = None
    stdinOnce: bool | None = None
    targetContainerName: str | None = None
    terminationMessagePath: str | None = None
    terminationMessagePolicy: str | None = None
    tty: bool | None = None
    volumeDevices: List[VolumeDevice] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'devicePath',
        },
    )
    volumeMounts: List[VolumeMount] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'mountPath',
        },
    )
    workingDir: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ProjectedVolumeSource(Loadable):
    defaultMode: int | None = None
    sources: List[VolumeProjection] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class Volume(Loadable):
    name: str
    awsElasticBlockStore: AWSElasticBlockStoreVolumeSource | None = None
    azureDisk: AzureDiskVolumeSource | None = None
    azureFile: AzureFileVolumeSource | None = None
    cephfs: CephFSVolumeSource | None = None
    cinder: CinderVolumeSource | None = None
    configMap: ConfigMapVolumeSource | None = None
    csi: CSIVolumeSource | None = None
    downwardAPI: DownwardAPIVolumeSource | None = None
    emptyDir: EmptyDirVolumeSource | None = None
    ephemeral: EphemeralVolumeSource | None = None
    fc: FCVolumeSource | None = None
    flexVolume: FlexVolumeSource | None = None
    flocker: FlockerVolumeSource | None = None
    gcePersistentDisk: GCEPersistentDiskVolumeSource | None = None
    gitRepo: GitRepoVolumeSource | None = None
    glusterfs: GlusterfsVolumeSource | None = None
    hostPath: HostPathVolumeSource | None = None
    image: ImageVolumeSource | None = None
    iscsi: ISCSIVolumeSource | None = None
    nfs: NFSVolumeSource | None = None
    persistentVolumeClaim: PersistentVolumeClaimVolumeSource | None = None
    photonPersistentDisk: PhotonPersistentDiskVolumeSource | None = None
    portworxVolume: PortworxVolumeSource | None = None
    projected: ProjectedVolumeSource | None = None
    quobyte: QuobyteVolumeSource | None = None
    rbd: RBDVolumeSource | None = None
    scaleIO: ScaleIOVolumeSource | None = None
    secret: SecretVolumeSource | None = None
    storageos: StorageOSVolumeSource | None = None
    vsphereVolume: VsphereVirtualDiskVolumeSource | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PodSpec(Loadable):
    activeDeadlineSeconds: int | None = None
    affinity: Affinity | None = None
    automountServiceAccountToken: bool | None = None
    containers: List[Container] = field(
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'name',
        }
    )
    dnsConfig: PodDNSConfig | None = None
    dnsPolicy: str | None = None
    enableServiceLinks: bool | None = None
    ephemeralContainers: List[EphemeralContainer] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'name',
        },
    )
    hostAliases: List[HostAlias] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'ip',
        },
    )
    hostIPC: bool | None = None
    hostNetwork: bool | None = None
    hostPID: bool | None = None
    hostUsers: bool | None = None
    hostname: str | None = None
    hostnameOverride: str | None = None
    imagePullSecrets: List[LocalObjectReference] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'name',
        },
    )
    initContainers: List[Container] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'name',
        },
    )
    nodeName: str | None = None
    nodeSelector: Dict[str, str] | None = None
    os: PodOS | None = None
    overhead: Dict[str, Quantity] | None = None
    preemptionPolicy: str | None = None
    priority: int | None = None
    priorityClassName: str | None = None
    readinessGates: List[PodReadinessGate] | None = None
    resourceClaims: List[PodResourceClaim] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge,retainKeys',
            'x-kubernetes-patch-merge-key': 'name',
        },
    )
    resources: ResourceRequirements | None = None
    restartPolicy: str | None = None
    runtimeClassName: str | None = None
    schedulerName: str | None = None
    schedulingGates: List[PodSchedulingGate] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'name',
        },
    )
    securityContext: PodSecurityContext | None = None
    serviceAccount: str | None = None
    serviceAccountName: str | None = None
    setHostnameAsFQDN: bool | None = None
    shareProcessNamespace: bool | None = None
    subdomain: str | None = None
    terminationGracePeriodSeconds: int | None = None
    tolerations: List[Toleration] | None = None
    topologySpreadConstraints: List[TopologySpreadConstraint] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'topologyKey',
        },
    )
    volumes: List[Volume] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge,retainKeys',
            'x-kubernetes-patch-merge-key': 'name',
        },
    )


@dataclass(slots=True, kw_only=True, frozen=True)
class PodTemplateSpec(Loadable):
    metadata: ObjectMeta | None = None
    spec: PodSpec | None = None



# generated by https://github.com/puzl-cloud/kubesdk
from .v1 import CrossVersionObjectReference, HorizontalPodAutoscaler, HorizontalPodAutoscalerList, HorizontalPodAutoscalerSpec, HorizontalPodAutoscalerStatus




__all__ = ['CrossVersionObjectReference', 'HorizontalPodAutoscaler', 'HorizontalPodAutoscalerList', 'HorizontalPodAutoscalerSpec', 'HorizontalPodAutoscalerStatus']




# generated by https://github.com/puzl-cloud/kubesdk
#   filename:  apis__rbac.authorization.k8s.io__v1_openapi.json

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime, timezone
from typing import ClassVar, List, Set

from kube_models.const import *
from kube_models.loader import *
from kube_models.loader import Loadable
from kube_models.resource import *

from ...apimachinery.pkg.apis.meta.v1 import LabelSelector, ListMeta, ObjectMeta


@dataclass(slots=True, kw_only=True, frozen=True)
class PolicyRule(Loadable):
    verbs: List[str]
    apiGroups: List[str] | None = None
    nonResourceURLs: List[str] | None = None
    resourceNames: List[str] | None = None
    resources: List[str] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class RoleRef(Loadable):
    apiGroup: str
    kind: str
    name: str


@dataclass(slots=True, kw_only=True, frozen=True)
class Subject(Loadable):
    kind: str
    name: str
    apiGroup: str | None = None
    namespace: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class AggregationRule(Loadable):
    clusterRoleSelectors: List[LabelSelector] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ClusterRole(K8sResource):
    aggregationRule: AggregationRule | None = None
    apiVersion: ClassVar[str] = 'rbac.authorization.k8s.io/v1'
    kind: ClassVar[str] = 'ClusterRole'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    rules: List[PolicyRule] | None = None
    plural_: ClassVar[str] = 'clusterroles'
    is_namespaced_: ClassVar[bool] = False
    group_: ClassVar[Optional[str]] = 'rbac.authorization.k8s.io'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class ClusterRoleBinding(K8sResource):
    roleRef: RoleRef
    apiVersion: ClassVar[str] = 'rbac.authorization.k8s.io/v1'
    kind: ClassVar[str] = 'ClusterRoleBinding'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    subjects: List[Subject] | None = None
    plural_: ClassVar[str] = 'clusterrolebindings'
    is_namespaced_: ClassVar[bool] = False
    group_: ClassVar[Optional[str]] = 'rbac.authorization.k8s.io'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class ClusterRoleBindingList(Loadable):
    items: List[ClusterRoleBinding]
    apiVersion: str = 'rbac.authorization.k8s.io/v1'
    kind: str = 'ClusterRoleBindingList'
    metadata: ListMeta = field(default_factory=ObjectMeta)


@dataclass(slots=True, kw_only=True, frozen=True)
class ClusterRoleList(Loadable):
    items: List[ClusterRole]
    apiVersion: str = 'rbac.authorization.k8s.io/v1'
    kind: str = 'ClusterRoleList'
    metadata: ListMeta = field(default_factory=ObjectMeta)


@dataclass(slots=True, kw_only=True, frozen=True)
class Role(K8sResource):
    apiVersion: ClassVar[str] = 'rbac.authorization.k8s.io/v1'
    kind: ClassVar[str] = 'Role'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    rules: List[PolicyRule] | None = None
    plural_: ClassVar[str] = 'roles'
    is_namespaced_: ClassVar[bool] = True
    group_: ClassVar[Optional[str]] = 'rbac.authorization.k8s.io'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class RoleBinding(K8sResource):
    roleRef: RoleRef
    apiVersion: ClassVar[str] = 'rbac.authorization.k8s.io/v1'
    kind: ClassVar[str] = 'RoleBinding'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    subjects: List[Subject] | None = None
    plural_: ClassVar[str] = 'rolebindings'
    is_namespaced_: ClassVar[bool] = True
    group_: ClassVar[Optional[str]] = 'rbac.authorization.k8s.io'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class RoleBindingList(Loadable):
    items: List[RoleBinding]
    apiVersion: str = 'rbac.authorization.k8s.io/v1'
    kind: str = 'RoleBindingList'
    metadata: ListMeta = field(default_factory=ObjectMeta)


@dataclass(slots=True, kw_only=True, frozen=True)
class RoleList(Loadable):
    items: List[Role]
    apiVersion: str = 'rbac.authorization.k8s.io/v1'
    kind: str = 'RoleList'
    metadata: ListMeta = field(default_factory=ObjectMeta)



from __future__ import annotations

import sys
import logging
import typing
from datetime import datetime
from base64 import b64encode
import json
from dataclasses import is_dataclass, fields, field, dataclass, Field
from typing import *
from types import UnionType

if sys.version_info < (3, 11):
    from typing_extensions import Self

from .const import *
from .registry import register_model


_CACHED_TYPES = {}

_LOAD_TYPES_ON_INIT = "__should_load"
_LOAD_LAZY_FIELD = "_lazy"
_LAZY_SRC_FIELD = "_lazy_src"


def _supports_lazy_load(obj: Any) -> bool:
    if not is_dataclass(obj):
        return False
    all_fields = fields(obj)
    return bool([f for f in all_fields if f.name == _LAZY_SRC_FIELD])


def _inject_lazy_load(obj_type: type[Any], kwargs: dict[str, Any], use_lazy: bool) -> dict[str, Any]:
    if _supports_lazy_load(obj_type) and use_lazy:
        return kwargs | {_LOAD_LAZY_FIELD: True, _LOAD_TYPES_ON_INIT: True}
    return kwargs


def _pep604_to_union(type_name: str) -> str:
    """
    Convert a PEP 604 union expression in string form (e.g. "str | None")
    into a typing-style Union[...] string (e.g. "Union[str, None]").

    - If there's no top-level '|' it returns the input unchanged.
    - If it's already of the form Union[...], it returns it unchanged.
    - Ignores '|' characters inside brackets/parentheses or quotes.
    """
    s = type_name.strip()
    if '|' not in s:
        return s

    def is_balanced(text: str) -> bool:
        stack, pairs = [], {')': '(', ']': '[', '}': '{'}
        for ch in text:
            if ch in '([{':
                stack.append(ch)
            elif ch in ')]}':
                if not stack or stack[-1] != pairs[ch]:
                    return False
                stack.pop()
        return not stack

    def strip_parens(x: str) -> str:
        y = x.strip()
        while y.startswith('(') and y.endswith(')') and is_balanced(y[1:-1]):
            y = y[1:-1].strip()
        return y

    s = strip_parens(s)

    def split_top_level_union(expr: str):
        parts, buf, stack = [], [], []
        pairs = {')': '(', ']': '[', '}': '{'}
        in_single = in_double = False
        prev = ''
        for ch in expr:
            if ch == "'" and not in_double and prev != '\\':
                in_single = not in_single
            elif ch == '"' and not in_single and prev != '\\':
                in_double = not in_double
            elif not in_single and not in_double:
                if ch in '([{':
                    stack.append(ch)
                elif ch in ')]}':
                    if stack and stack[-1] == pairs[ch]:
                        stack.pop()
                elif ch == '|' and not stack:
                    parts.append(strip_parens(''.join(buf).strip()))
                    buf = []
                    prev = ch
                    continue
            buf.append(ch)
            prev = ch
        parts.append(strip_parens(''.join(buf).strip()))
        return parts

    parts = split_top_level_union(s)
    if len(parts) == 1:
        return s
    return f"Union[{', '.join(parts)}]"


def _extract_real_type(tp: Any) -> Tuple[Tuple[Any], bool]:
    """
    Unwraps Optional[T] and Union[..., NoneType], returning (base_type, is_optional).

      - Optional[T] / T | None      -> (T, True)
      - Union[A, B, …]              -> (Union[A, B, …] (None removed), True if None present)
      - Everything else (incl. List[T], Dict[K, V], etc.) -> (original type, False)

    Notes:
      * Preserves complex typing objects where possible (e.g., Union[int, str]).
      * Does NOT normalize inside non-Union containers (e.g., List[Optional[int]] stays as-is).
      * No eval() used.
    """
    origin = get_origin(tp)

    # Handle Union / PEP 604 unions
    if origin is Union or (getattr(UnionType, "__mro__", ()) and origin is UnionType):
        opt = False
        new_branches = []
        for arg in get_args(tp):
            if arg is type(None):  # NoneType
                opt = True
                continue
            base_type, inner_opt = _extract_real_type(arg)
            opt = opt or inner_opt
            new_branches.append(base_type)

        if not new_branches:
            # Union[None] -> (NoneType, True)
            return (type(None),), True

        # Collapse single branch; otherwise, rebuild Union without None
        # if len(new_branches) == 1:
        #     return new_branches[0], opt
        flat = tuple(x for a in new_branches for x in (a if isinstance(a, tuple) else (a,)))
        return flat, opt

    # Everything else: leave untouched
    return (tp,), False


def _evaluate_field_by_meta(field_value: Any, metadata: Mapping) -> Any:
    try:
        return metadata.get('decoder')(field_value)
    except Exception as e:
        raise ValueError(e)


def _evaluate_value(field_type: Any, field_value: Any, use_lazy: bool = False) -> Any:
    try:
        # Preserve original arg for later
        passed_field_type = field_type

        # Use cache to avoid useless recursions every time
        if passed_field_type in _CACHED_TYPES:
            cached_field_type = _CACHED_TYPES[passed_field_type]

            # We cache None for all scalars
            if cached_field_type is None:
                return field_value
            try:
                injected_kw = _inject_lazy_load(cached_field_type, field_value, use_lazy)
                return cached_field_type(**injected_kw)
            except Exception:
                return field_value

        if isinstance(field_type, str):
            # Convert Unions if any
            field_type = _pep604_to_union(field_type)
            try:
                field_type = eval(field_type)
            except Exception as e:
                return field_value

        field_type_origin = get_origin(field_type)
        if any(field_type_origin is scalar for scalar in SCALAR_TYPES):
            # Cache None for scalars
            _CACHED_TYPES[passed_field_type] = None
            return field_value

        # Check if the field type is a dataclass and if kwargs value is a dict
        if isinstance(field_type, type) and is_dataclass(field_type) and isinstance(field_value, dict):
            _CACHED_TYPES[passed_field_type] = field_type
            injected_kw = _inject_lazy_load(field_type, field_value, use_lazy)
            return field_type(**injected_kw)

        # Handle lists of dataclasses
        elif field_type_origin is list:
            value_type = get_args(field_type)
            if not value_type:
                return field_value
            list_arg_type = value_type[0]
            if is_dataclass(list_arg_type) and isinstance(field_value, list):
                return [
                    list_arg_type(**_inject_lazy_load(list_arg_type, item, use_lazy)) if isinstance(item, dict)
                    else item for item in field_value
                ]

            # Cache untyped general list
            _CACHED_TYPES[passed_field_type] = None
            return field_value

        # Handle Dict[SomeType, SomeDataClass]
        elif field_type_origin is dict:
            dict_kv_types = get_args(field_type)
            if not dict_kv_types:
                return field_value
            key_type, value_type = dict_kv_types
            if is_dataclass(value_type) and isinstance(field_value, dict):
                return {
                    key: value_type(**_inject_lazy_load(value_type, val, use_lazy)) if isinstance(val, dict)
                    else val for key, val in field_value.items()
                }

            # Cache untyped general dict
            _CACHED_TYPES[passed_field_type] = None
            return field_value

        # Handle everything else including Optional types
        else:
            real_type, field_type_is_optional = _extract_real_type(field_type)
            for item in real_type:
                try:
                    if any(item is scalar for scalar in SCALAR_TYPES):
                        # Cache None for scalars
                        res = field_value
                        _CACHED_TYPES[passed_field_type] = None
                    else:
                        injected_kw = _inject_lazy_load(item, field_value, use_lazy)
                        res = item(**injected_kw)
                        _CACHED_TYPES[passed_field_type] = item
                    return res
                except Exception as e:
                    pass

            # ToDo: Actually, we must do further checks for origin of each real_type here, not just take the first one
            # If we were not able to find any type, just go with the first one
            real_type = real_type[0]

            # If we have complex type still, decompose it further
            if "[" in str(real_type):
                return _evaluate_value(real_type, field_value, use_lazy)

            if isinstance(real_type, type):
                try:
                    injected_kw = _inject_lazy_load(real_type, field_value, use_lazy)
                    res = real_type(**injected_kw)
                    _CACHED_TYPES[passed_field_type] = real_type
                    return res
                except (SyntaxError, NameError, TypeError):
                    pass

            try:
                real_type_origin = get_origin(real_type)
                if any(real_type_origin is scalar for scalar in SCALAR_TYPES):
                    # Cache None for scalars
                    res = field_value
                    _CACHED_TYPES[passed_field_type] = None
                else:
                    injected_kw = _inject_lazy_load(real_type_origin, field_value, use_lazy)
                    res = real_type_origin(**injected_kw)
                    _CACHED_TYPES[passed_field_type] = real_type_origin
                return res
            except (SyntaxError, NameError, TypeError):
                # Give up here. It could be enum or some other complex scalar, just leave it as is.
                _CACHED_TYPES[passed_field_type] = None
                return field_value
            except BaseException:
                raise
    except BaseException as e:
        logging.critical(
            f"Unexpected error in _evaluate_value for type {field_type} and value {field_value}. Error: {e}")
        raise


def k8s_timestamp_field(**kwargs) -> Field:
    return field(**kwargs, metadata={
        "decoder": lambda iso_date: datetime.fromisoformat(iso_date) if type(iso_date) is str else iso_date,
        "encoder": lambda date: date.isoformat("T", "seconds").replace("+00:00", "Z")
        if type(date) is datetime else date
    })


def _to_immutable(v: Any) -> Any:
    """Best-effort stable, hashable transform for nested structures."""
    try:
        hash(v)
        return v
    except TypeError:
        if isinstance(v, dict):
            return tuple(sorted((k, _to_immutable(val)) for k, val in v.items()))
        if isinstance(v, (list, tuple)):
            return tuple(_to_immutable(x) for x in v)
        if isinstance(v, set):
            return tuple(sorted(_to_immutable(x) for x in v))
        # Fallback for user types
        return repr(v)


class _LoadableMeta(type):
    """
    A metaclass for dataclasses to automatically decode fields during initialization
    based on the metadata configuration and typing.

    This metaclass intercepts the initialization arguments and applies the specified
    decoder functions (if any) from the field metadata before passing the arguments
    to the original ``__init__`` method.
    """
    def __init__(cls, *args, **kwargs):
        super().__init__(*args, **kwargs)
        cls._ensure_typing_import()
        register_model(cls)

    def __call__(cls, *args, **kwargs):
        new_kw: Dict[str, Any] = {}
        supports_lazy = issubclass(cls, Loadable)
        lazy_requested = kwargs.get(_LOAD_LAZY_FIELD)
        should_use_lazy = supports_lazy and lazy_requested
        if not getattr(cls, "__lazy_methods_patched", False):
            cls.__eq__ = Loadable.__eq__
            cls.__hash__ = Loadable.__hash__
            cls.__lazy_methods_patched = True

        if not should_use_lazy:
            if not kwargs.get(_LOAD_TYPES_ON_INIT):
                return super().__call__(*args, **kwargs)

            all_fields = fields(cls)
            if not hasattr(cls, "__lazy_type_hints"):
                cls.__lazy_type_hints = get_type_hints(cls)

            for f in all_fields:
                if not f.init or f.name not in kwargs:
                    continue

                # metadata-based decoder
                if "decoder" in f.metadata:
                    try:
                        kwargs[f.name] = _evaluate_field_by_meta(kwargs[f.name], f.metadata)
                        continue
                    except ValueError as e:
                        logging.error(
                            f"Got invalid decoder for {f.name} in class {cls.__name__}. "
                            f"Value was passed as is. Error: {e}"
                        )

                field_type = cls.__lazy_type_hints.get(f.name, f.type)
                if any(field_type is scalar for scalar in SCALAR_TYPES):
                    continue
                kwargs[f.name] = _evaluate_value(field_type, kwargs[f.name], use_lazy=False)

            new_kw = {
                k: v
                for k, v in kwargs.items()
                if k not in (_LOAD_LAZY_FIELD, _LOAD_TYPES_ON_INIT)
            }
        else:
            # Lazy path: store original kwargs as _lazy_src, set list/dict fields to None
            for k, v in kwargs.items():
                if k == _LOAD_TYPES_ON_INIT:
                    continue
                new_kw[k] = v if type(v) not in [list, dict] else None

            new_kw |= {_LAZY_SRC_FIELD: kwargs, _LOAD_LAZY_FIELD: True}

        return super().__call__(*args, **new_kw)


@dataclass(slots=True, kw_only=True, frozen=True)
class Loadable(metaclass=_LoadableMeta):
    """
    This model supports lazy loading of complex nested types avoiding unnecessary heavy recursions.
    """
    _lazy_src: Dict = field(default_factory=dict, repr=False)
    _lazy: bool = field(default=False, repr=False)

    @classmethod
    def _ensure_typing_import(cls) -> None:
        module = sys.modules.get(cls.__module__)
        if not module:
            return
        _globals = module.__dict__
        if _globals.get("__KUBESDK_TYPING_STAR_DONE"):
            return

        # from typing import * into class module
        for name in getattr(typing, "__all__", ()):
            _globals.setdefault(name, getattr(typing, name))
        _globals["__KUBESDK_TYPING_STAR_DONE"] = True

    @classmethod
    def from_dict(cls, src: dict[str, Any], lazy: bool = True) -> Self:
        params = dict(src)
        params[_LOAD_LAZY_FIELD] = lazy
        params[_LOAD_TYPES_ON_INIT] = True
        return cls(**params)

    def __getattribute__(self, name: str) -> Any:
        """
        Overrides the default attribute access behavior to implement lazy construction of Python types in a runtime.
        This method ensures that all the fields are only computed when they are accessed for the first time.

        If these attributes are not set during initialization (i.e., they are None),
        this method triggers their class constructor. This approach allows the class to behave like a normal
        frozen dataclass, where attributes can be accessed directly without the user needing to be aware
        of the lazy loading mechanism.
        """

        # Lazy load release data from body, if it's not set yet. We're interested in the following fields only.
        if name in [_LAZY_SRC_FIELD, _LOAD_LAZY_FIELD] or name.startswith("__") \
                or name not in getattr(self.__class__, "__dataclass_fields__", {}):
            # Never call super() here:
            # slotted dataclass have no __class__ attribute, which will end with infinite recursion
            return object.__getattribute__(self, name)

        # Check if we even want lazy load for this instance
        if not self._lazy:
            return object.__getattribute__(self, name)

        current_value = object.__getattribute__(self, name)
        if current_value is not None:
            return current_value
        src_value = self._lazy_src.get(name)
        if current_value == src_value:
            return current_value

        field_value, decoded = None, False
        f = self.__class__.__dataclass_fields__[name]
        if "decoder" in f.metadata:
            try:
                field_value = _evaluate_field_by_meta(src_value, f.metadata)
                decoded = True
            except ValueError as e:
                logging.error(f"Got invalid decoder for {f.name} in class {self.__class__.__name__}. "
                              f"Value was passed as is. Error: {e}")
                pass

        if not decoded:
            # Use cached type hints if available
            cls = self.__class__
            hints = getattr(cls, "__lazy_type_hints", None)
            if hints is None:
                globalns = vars(sys.modules[cls.__module__])
                hints = get_type_hints(cls, globalns=globalns, include_extras=True)
                setattr(cls, "__lazy_type_hints", hints)

            field_type = hints[name]
            field_value = _evaluate_value(field_type, src_value, use_lazy=self._lazy)
        object.__setattr__(self, name, field_value)
        return object.__getattribute__(self, name)

    def _realize_all(self) -> None:
        """Force load of all dataclass fields"""
        for n in getattr(self.__class__, "__dataclass_fields__", {}):
            if n in [_LAZY_SRC_FIELD, _LOAD_LAZY_FIELD]:
                continue
            _ = getattr(self, n)

    def __getstate__(self):
        """
        Make deepcopy/pickle see realized state without implementing our own deepcopy.
        """
        self._realize_all()

        # build state dict manually (slots instance has no __dict__)
        cls = object.__getattribute__(self, "__class__")
        state = {}
        for f in fields(cls):
            state[f.name] = object.__getattribute__(self, f.name)
        return state

    def __setstate__(self, state):
        # Restore state (allowed for frozen via object.__setattr__)
        for k, v in state.items():
            object.__setattr__(self, k, v)

    def __eq__(self, other: Any) -> bool:
        if self is other:
            return True
        if other.__class__ is not self.__class__:
            return NotImplemented
        return self.to_dict() == other.to_dict()

    def __hash__(self) -> int:
        """
        Align hash with equality: hash over realized public fields only.
        Avoids mismatch where two objects are == but have different hashes.
        """
        self._realize_all()
        public_items = []
        for n in getattr(self.__class__, "__dataclass_fields__", {}):
            if n in [_LAZY_SRC_FIELD, _LOAD_LAZY_FIELD]:
                continue
            public_items.append((n, _to_immutable(getattr(self, n))))
        return hash(tuple(public_items))

    def to_dict(self) -> dict[str, Any]:
        result = {}
        for f in fields(self):
            # Skip all private fields
            if f.name.startswith("_"):
                continue

            # Skip dataclass fields with the 'exclude_from_dict' flag in metadata
            if f.metadata.get(EXCLUDE_FIELD_META_KEY):
                continue

            value = getattr(self, f.name)
            encoder = f.metadata.get('encoder')
            value = encoder(value) if encoder else value

            # Recursively call to_dict if the field is a dataclass instance
            if is_dataclass(value):
                value = value.to_dict()
            elif isinstance(value, list) and all(is_dataclass(item) for item in value):
                value = [item.to_dict() for item in value]
            elif isinstance(value, dict):
                new_value = {}
                for k, v in value.items():
                    new_value[k] = v.to_dict() if is_dataclass(v) else v
                value = new_value

            result[f.name] = value
        return result



# generated by https://github.com/puzl-cloud/kubesdk
#   filename:  apis__apps__v1_openapi.json

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime, timezone
from typing import ClassVar, List, Set

from kube_models.const import *
from kube_models.loader import *
from kube_models.loader import Loadable
from kube_models.resource import *

from ...apimachinery.pkg.apis.meta.v1 import LabelSelector, ListMeta, ObjectMeta, Time
from ...apimachinery.pkg.runtime import RawExtension
from ...apimachinery.pkg.util.intstr import IntOrString
from ..core.v1 import PersistentVolumeClaim, PodTemplateSpec


@dataclass(slots=True, kw_only=True, frozen=True)
class StatefulSetOrdinals(Loadable):
    start: int | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class StatefulSetPersistentVolumeClaimRetentionPolicy(Loadable):
    whenDeleted: str | None = None
    whenScaled: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class DaemonSetCondition(Loadable):
    status: str
    type: str
    lastTransitionTime: Time | None = None
    message: str | None = None
    reason: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class DaemonSetStatus(Loadable):
    currentNumberScheduled: int
    desiredNumberScheduled: int
    numberMisscheduled: int
    numberReady: int
    collisionCount: int | None = None
    conditions: List[DaemonSetCondition] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'type',
        },
    )
    numberAvailable: int | None = None
    numberUnavailable: int | None = None
    observedGeneration: int | None = None
    updatedNumberScheduled: int | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class DeploymentCondition(Loadable):
    status: str
    type: str
    lastTransitionTime: Time | None = None
    lastUpdateTime: Time | None = None
    message: str | None = None
    reason: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class DeploymentStatus(Loadable):
    availableReplicas: int | None = None
    collisionCount: int | None = None
    conditions: List[DeploymentCondition] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'type',
        },
    )
    observedGeneration: int | None = None
    readyReplicas: int | None = None
    replicas: int | None = None
    terminatingReplicas: int | None = None
    unavailableReplicas: int | None = None
    updatedReplicas: int | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ReplicaSetCondition(Loadable):
    status: str
    type: str
    lastTransitionTime: Time | None = None
    message: str | None = None
    reason: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ReplicaSetStatus(Loadable):
    replicas: int
    availableReplicas: int | None = None
    conditions: List[ReplicaSetCondition] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'type',
        },
    )
    fullyLabeledReplicas: int | None = None
    observedGeneration: int | None = None
    readyReplicas: int | None = None
    terminatingReplicas: int | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class RollingUpdateDaemonSet(Loadable):
    maxSurge: IntOrString | None = None
    maxUnavailable: IntOrString | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class RollingUpdateDeployment(Loadable):
    maxSurge: IntOrString | None = None
    maxUnavailable: IntOrString | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class RollingUpdateStatefulSetStrategy(Loadable):
    maxUnavailable: IntOrString | None = None
    partition: int | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class StatefulSetCondition(Loadable):
    status: str
    type: str
    lastTransitionTime: Time | None = None
    message: str | None = None
    reason: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class StatefulSetStatus(Loadable):
    replicas: int
    availableReplicas: int | None = None
    collisionCount: int | None = None
    conditions: List[StatefulSetCondition] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'type',
        },
    )
    currentReplicas: int | None = None
    currentRevision: str | None = None
    observedGeneration: int | None = None
    readyReplicas: int | None = None
    updateRevision: str | None = None
    updatedReplicas: int | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class StatefulSetUpdateStrategy(Loadable):
    rollingUpdate: RollingUpdateStatefulSetStrategy | None = None
    type: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ControllerRevision(K8sResource):
    revision: int
    apiVersion: ClassVar[str] = 'apps/v1'
    data: RawExtension | None = None
    kind: ClassVar[str] = 'ControllerRevision'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    plural_: ClassVar[str] = 'controllerrevisions'
    is_namespaced_: ClassVar[bool] = True
    group_: ClassVar[Optional[str]] = 'apps'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class ControllerRevisionList(Loadable):
    items: List[ControllerRevision]
    apiVersion: str = 'apps/v1'
    kind: str = 'ControllerRevisionList'
    metadata: ListMeta = field(default_factory=ObjectMeta)


@dataclass(slots=True, kw_only=True, frozen=True)
class DaemonSetUpdateStrategy(Loadable):
    rollingUpdate: RollingUpdateDaemonSet | None = None
    type: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class DeploymentStrategy(Loadable):
    rollingUpdate: RollingUpdateDeployment | None = None
    type: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class DaemonSetSpec(Loadable):
    selector: LabelSelector
    template: PodTemplateSpec
    minReadySeconds: int | None = None
    revisionHistoryLimit: int | None = None
    updateStrategy: DaemonSetUpdateStrategy | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class DeploymentSpec(Loadable):
    selector: LabelSelector
    template: PodTemplateSpec
    minReadySeconds: int | None = None
    paused: bool | None = None
    progressDeadlineSeconds: int | None = None
    replicas: int | None = None
    revisionHistoryLimit: int | None = None
    strategy: DeploymentStrategy = field(
        default_factory=dict, metadata={'x-kubernetes-patch-strategy': 'retainKeys'}
    )


@dataclass(slots=True, kw_only=True, frozen=True)
class ReplicaSetSpec(Loadable):
    selector: LabelSelector
    minReadySeconds: int | None = None
    replicas: int | None = None
    template: PodTemplateSpec | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class StatefulSetSpec(Loadable):
    selector: LabelSelector
    template: PodTemplateSpec
    minReadySeconds: int | None = None
    ordinals: StatefulSetOrdinals | None = None
    persistentVolumeClaimRetentionPolicy: (
        StatefulSetPersistentVolumeClaimRetentionPolicy | None
    ) = None
    podManagementPolicy: str | None = None
    replicas: int | None = None
    revisionHistoryLimit: int | None = None
    serviceName: str | None = None
    updateStrategy: StatefulSetUpdateStrategy | None = None
    volumeClaimTemplates: List[PersistentVolumeClaim] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class DaemonSet(K8sResource):
    apiVersion: ClassVar[str] = 'apps/v1'
    kind: ClassVar[str] = 'DaemonSet'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    spec: DaemonSetSpec | None = None
    status: DaemonSetStatus | None = None
    plural_: ClassVar[str] = 'daemonsets'
    is_namespaced_: ClassVar[bool] = True
    group_: ClassVar[Optional[str]] = 'apps'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class DaemonSetList(Loadable):
    items: List[DaemonSet]
    apiVersion: str = 'apps/v1'
    kind: str = 'DaemonSetList'
    metadata: ListMeta = field(default_factory=ObjectMeta)


@dataclass(slots=True, kw_only=True, frozen=True)
class Deployment(K8sResource):
    apiVersion: ClassVar[str] = 'apps/v1'
    kind: ClassVar[str] = 'Deployment'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    spec: DeploymentSpec | None = None
    status: DeploymentStatus | None = None
    plural_: ClassVar[str] = 'deployments'
    is_namespaced_: ClassVar[bool] = True
    group_: ClassVar[Optional[str]] = 'apps'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class DeploymentList(Loadable):
    items: List[Deployment]
    apiVersion: str = 'apps/v1'
    kind: str = 'DeploymentList'
    metadata: ListMeta = field(default_factory=ObjectMeta)


@dataclass(slots=True, kw_only=True, frozen=True)
class ReplicaSet(K8sResource):
    apiVersion: ClassVar[str] = 'apps/v1'
    kind: ClassVar[str] = 'ReplicaSet'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    spec: ReplicaSetSpec | None = None
    status: ReplicaSetStatus | None = None
    plural_: ClassVar[str] = 'replicasets'
    is_namespaced_: ClassVar[bool] = True
    group_: ClassVar[Optional[str]] = 'apps'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class ReplicaSetList(Loadable):
    items: List[ReplicaSet]
    apiVersion: str = 'apps/v1'
    kind: str = 'ReplicaSetList'
    metadata: ListMeta = field(default_factory=ObjectMeta)


@dataclass(slots=True, kw_only=True, frozen=True)
class StatefulSet(K8sResource):
    apiVersion: ClassVar[str] = 'apps/v1'
    kind: ClassVar[str] = 'StatefulSet'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    spec: StatefulSetSpec | None = None
    status: StatefulSetStatus | None = None
    plural_: ClassVar[str] = 'statefulsets'
    is_namespaced_: ClassVar[bool] = True
    group_: ClassVar[Optional[str]] = 'apps'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class StatefulSetList(Loadable):
    items: List[StatefulSet]
    apiVersion: str = 'apps/v1'
    kind: str = 'StatefulSetList'
    metadata: ListMeta = field(default_factory=ObjectMeta)



# generated by https://github.com/puzl-cloud/kubesdk
from .v1 import CertificateSigningRequest, CertificateSigningRequestCondition, CertificateSigningRequestList, CertificateSigningRequestSpec, CertificateSigningRequestStatus




__all__ = ['CertificateSigningRequest', 'CertificateSigningRequestCondition', 'CertificateSigningRequestList', 'CertificateSigningRequestSpec', 'CertificateSigningRequestStatus']




# generated by https://github.com/puzl-cloud/kubesdk
from .v1alpha1 import IPAddress, IPAddressList, IPAddressSpec, ParentReference, ServiceCIDR, ServiceCIDRList, ServiceCIDRSpec, ServiceCIDRStatus




__all__ = ['IPAddress', 'IPAddressList', 'IPAddressSpec', 'ParentReference', 'ServiceCIDR', 'ServiceCIDRList', 'ServiceCIDRSpec', 'ServiceCIDRStatus']




# generated by https://github.com/puzl-cloud/kubesdk
#   filename:  apis__resource.k8s.io__v1alpha3_openapi.json

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime, timezone
from typing import ClassVar, List, Set

from kube_models.const import *
from kube_models.loader import *
from kube_models.loader import Loadable
from kube_models.resource import *

from ...apimachinery.pkg.apis.meta.v1 import ListMeta, ObjectMeta, Time


@dataclass(slots=True, kw_only=True, frozen=True)
class CELDeviceSelector(Loadable):
    expression: str


@dataclass(slots=True, kw_only=True, frozen=True)
class DeviceSelector(Loadable):
    cel: CELDeviceSelector | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class DeviceTaintSelector(Loadable):
    device: str | None = None
    deviceClassName: str | None = None
    driver: str | None = None
    pool: str | None = None
    selectors: List[DeviceSelector] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class DeviceTaint(Loadable):
    effect: str
    key: str
    timeAdded: Time | None = None
    value: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class DeviceTaintRuleSpec(Loadable):
    taint: DeviceTaint
    deviceSelector: DeviceTaintSelector | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class DeviceTaintRule(K8sResource):
    spec: DeviceTaintRuleSpec
    apiVersion: ClassVar[str] = 'resource.k8s.io/v1alpha3'
    kind: ClassVar[str] = 'DeviceTaintRule'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    plural_: ClassVar[str] = 'devicetaintrules'
    is_namespaced_: ClassVar[bool] = False
    group_: ClassVar[Optional[str]] = 'resource.k8s.io'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class DeviceTaintRuleList(Loadable):
    items: List[DeviceTaintRule]
    apiVersion: str = 'resource.k8s.io/v1alpha3'
    kind: str = 'DeviceTaintRuleList'
    metadata: ListMeta = field(default_factory=ObjectMeta)



# generated by https://github.com/puzl-cloud/kubesdk
#   filename:  apis__certificates.k8s.io__v1beta1_openapi.json

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime, timezone
from typing import ClassVar, List, Set

from kube_models.const import *
from kube_models.loader import *
from kube_models.loader import Loadable
from kube_models.resource import *

from ...apimachinery.pkg.apis.meta.v1 import ListMeta, ObjectMeta


@dataclass(slots=True, kw_only=True, frozen=True)
class ClusterTrustBundleSpec(Loadable):
    trustBundle: str
    signerName: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ClusterTrustBundle(K8sResource):
    spec: ClusterTrustBundleSpec
    apiVersion: ClassVar[str] = 'certificates.k8s.io/v1beta1'
    kind: ClassVar[str] = 'ClusterTrustBundle'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    plural_: ClassVar[str] = 'clustertrustbundles'
    is_namespaced_: ClassVar[bool] = False
    group_: ClassVar[Optional[str]] = 'certificates.k8s.io'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class ClusterTrustBundleList(Loadable):
    items: List[ClusterTrustBundle]
    apiVersion: str = 'certificates.k8s.io/v1beta1'
    kind: str = 'ClusterTrustBundleList'
    metadata: ListMeta = field(default_factory=ObjectMeta)



# generated by https://github.com/puzl-cloud/kubesdk
from .v1 import AuditAnnotation, ExpressionWarning, MatchCondition, MatchResources, MutatingWebhook, MutatingWebhookConfiguration, MutatingWebhookConfigurationList, NamedRuleWithOperations, ParamKind, ParamRef, RuleWithOperations, ServiceReference, TypeChecking, ValidatingAdmissionPolicy, ValidatingAdmissionPolicyBinding, ValidatingAdmissionPolicyBindingList, ValidatingAdmissionPolicyBindingSpec, ValidatingAdmissionPolicyList, ValidatingAdmissionPolicySpec, ValidatingAdmissionPolicyStatus, ValidatingWebhook, ValidatingWebhookConfiguration, ValidatingWebhookConfigurationList, Validation, Variable, WebhookClientConfig




__all__ = ['AuditAnnotation', 'ExpressionWarning', 'MatchCondition', 'MatchResources', 'MutatingWebhook', 'MutatingWebhookConfiguration', 'MutatingWebhookConfigurationList', 'NamedRuleWithOperations', 'ParamKind', 'ParamRef', 'RuleWithOperations', 'ServiceReference', 'TypeChecking', 'ValidatingAdmissionPolicy', 'ValidatingAdmissionPolicyBinding', 'ValidatingAdmissionPolicyBindingList', 'ValidatingAdmissionPolicyBindingSpec', 'ValidatingAdmissionPolicyList', 'ValidatingAdmissionPolicySpec', 'ValidatingAdmissionPolicyStatus', 'ValidatingWebhook', 'ValidatingWebhookConfiguration', 'ValidatingWebhookConfigurationList', 'Validation', 'Variable', 'WebhookClientConfig']




from __future__ import annotations

from dataclasses import MISSING
from typing import Any, Mapping, Optional

ModelKey = tuple[str, str]  # apiVersion, kind
ALL_RESOURCES: dict[ModelKey, type[Any]] = {}


def maybe_get_model_key(model_class: type[Any]) -> Optional[ModelKey]:
    """Return (apiVersion, kind) if both exist as strings."""

    api_version = getattr(model_class, "apiVersion", None)
    kind = getattr(model_class, "kind", None)
    if isinstance(api_version, str) and isinstance(kind, str):
        return api_version, kind

    dataclass_fields = getattr(model_class, "__dataclass_fields__", None)
    if not isinstance(dataclass_fields, dict):
        return None

    api_version_field = dataclass_fields.get("apiVersion")
    kind_field = dataclass_fields.get("kind")

    def get_string_default(field_object: Any) -> str | None:
        if field_object is None:
            return None

        default_value = getattr(field_object, "default", MISSING)
        if default_value is MISSING:
            default_factory = getattr(field_object, "default_factory", MISSING)
            if default_factory is not MISSING:
                default_value = default_factory()

        return default_value if isinstance(default_value, str) else None

    api_version_default = get_string_default(api_version_field)
    kind_default = get_string_default(kind_field)
    if api_version_default is None or kind_default is None:
        return None

    return api_version_default, kind_default


def register_model(model_class: type[Any]) -> None:
    """Register a model class in the global registry, if it has a resolvable key."""
    model_key = maybe_get_model_key(model_class)
    if model_key is not None:
        ALL_RESOURCES.setdefault(model_key, model_class)


def get_model(api_version: str, kind: str) -> type[Any] | None:
    """Look up a model class by (apiVersion, kind)."""
    return ALL_RESOURCES.get((api_version, kind))


def get_model_by_body(body: Mapping[str, Any]) -> type[Any] | None:
    """Look up a model class by reading apiVersion/kind out of a Kubernetes-style dict."""
    api_version_value = body.get("apiVersion")
    kind_value = body.get("kind")

    if not isinstance(api_version_value, str) or not isinstance(kind_value, str):
        return None

    return get_model(api_version_value, kind_value)



# generated by https://github.com/puzl-cloud/kubesdk
#   filename:  apis__batch__v1_openapi.json

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime, timezone
from typing import ClassVar, List, Set

from kube_models.const import *
from kube_models.loader import *
from kube_models.loader import Loadable
from kube_models.resource import *

from ...apimachinery.pkg.apis.meta.v1 import LabelSelector, ListMeta, ObjectMeta, Time
from ..core.v1 import ObjectReference, PodTemplateSpec


@dataclass(slots=True, kw_only=True, frozen=True)
class PodFailurePolicyOnExitCodesRequirement(Loadable):
    operator: str
    values: List[int]
    containerName: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PodFailurePolicyOnPodConditionsPattern(Loadable):
    status: str
    type: str


@dataclass(slots=True, kw_only=True, frozen=True)
class PodFailurePolicyRule(Loadable):
    action: str
    onExitCodes: PodFailurePolicyOnExitCodesRequirement | None = None
    onPodConditions: List[PodFailurePolicyOnPodConditionsPattern] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class SuccessPolicyRule(Loadable):
    succeededCount: int | None = None
    succeededIndexes: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class UncountedTerminatedPods(Loadable):
    failed: List[str] | None = None
    succeeded: List[str] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class CronJobStatus(Loadable):
    active: List[ObjectReference] | None = None
    lastScheduleTime: Time | None = None
    lastSuccessfulTime: Time | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class JobCondition(Loadable):
    status: str
    type: str
    lastProbeTime: Time | None = None
    lastTransitionTime: Time | None = None
    message: str | None = None
    reason: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class JobStatus(Loadable):
    active: int | None = None
    completedIndexes: str | None = None
    completionTime: Time | None = None
    conditions: List[JobCondition] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'type',
        },
    )
    failed: int | None = None
    failedIndexes: str | None = None
    ready: int | None = None
    startTime: Time | None = None
    succeeded: int | None = None
    terminating: int | None = None
    uncountedTerminatedPods: UncountedTerminatedPods | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PodFailurePolicy(Loadable):
    rules: List[PodFailurePolicyRule]


@dataclass(slots=True, kw_only=True, frozen=True)
class SuccessPolicy(Loadable):
    rules: List[SuccessPolicyRule]


@dataclass(slots=True, kw_only=True, frozen=True)
class JobSpec(Loadable):
    template: PodTemplateSpec
    activeDeadlineSeconds: int | None = None
    backoffLimit: int | None = None
    backoffLimitPerIndex: int | None = None
    completionMode: str | None = None
    completions: int | None = None
    managedBy: str | None = None
    manualSelector: bool | None = None
    maxFailedIndexes: int | None = None
    parallelism: int | None = None
    podFailurePolicy: PodFailurePolicy | None = None
    podReplacementPolicy: str | None = None
    selector: LabelSelector | None = None
    successPolicy: SuccessPolicy | None = None
    suspend: bool | None = None
    ttlSecondsAfterFinished: int | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class JobTemplateSpec(Loadable):
    metadata: ObjectMeta | None = None
    spec: JobSpec | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class CronJobSpec(Loadable):
    jobTemplate: JobTemplateSpec
    schedule: str
    concurrencyPolicy: str | None = None
    failedJobsHistoryLimit: int | None = None
    startingDeadlineSeconds: int | None = None
    successfulJobsHistoryLimit: int | None = None
    suspend: bool | None = None
    timeZone: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class Job(K8sResource):
    apiVersion: ClassVar[str] = 'batch/v1'
    kind: ClassVar[str] = 'Job'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    spec: JobSpec | None = None
    status: JobStatus | None = None
    plural_: ClassVar[str] = 'jobs'
    is_namespaced_: ClassVar[bool] = True
    group_: ClassVar[Optional[str]] = 'batch'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class JobList(Loadable):
    items: List[Job]
    apiVersion: str = 'batch/v1'
    kind: str = 'JobList'
    metadata: ListMeta = field(default_factory=ObjectMeta)


@dataclass(slots=True, kw_only=True, frozen=True)
class CronJob(K8sResource):
    apiVersion: ClassVar[str] = 'batch/v1'
    kind: ClassVar[str] = 'CronJob'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    spec: CronJobSpec | None = None
    status: CronJobStatus | None = None
    plural_: ClassVar[str] = 'cronjobs'
    is_namespaced_: ClassVar[bool] = True
    group_: ClassVar[Optional[str]] = 'batch'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class CronJobList(Loadable):
    items: List[CronJob]
    apiVersion: str = 'batch/v1'
    kind: str = 'CronJobList'
    metadata: ListMeta = field(default_factory=ObjectMeta)



# generated by https://github.com/puzl-cloud/kubesdk
from .v1alpha1 import AllocationResult, PodScheduling, PodSchedulingList, PodSchedulingSpec, PodSchedulingStatus, ResourceClaim, ResourceClaimConsumerReference, ResourceClaimList, ResourceClaimParametersReference, ResourceClaimSchedulingStatus, ResourceClaimSpec, ResourceClaimStatus, ResourceClaimTemplate, ResourceClaimTemplateList, ResourceClaimTemplateSpec, ResourceClass, ResourceClassList, ResourceClassParametersReference




__all__ = ['AllocationResult', 'PodScheduling', 'PodSchedulingList', 'PodSchedulingSpec', 'PodSchedulingStatus', 'ResourceClaim', 'ResourceClaimConsumerReference', 'ResourceClaimList', 'ResourceClaimParametersReference', 'ResourceClaimSchedulingStatus', 'ResourceClaimSpec', 'ResourceClaimStatus', 'ResourceClaimTemplate', 'ResourceClaimTemplateList', 'ResourceClaimTemplateSpec', 'ResourceClass', 'ResourceClassList', 'ResourceClassParametersReference']




# generated by https://github.com/puzl-cloud/kubesdk
from .v1 import CSIDriver, CSIDriverList, CSIDriverSpec, CSINode, CSINodeDriver, CSINodeList, CSINodeSpec, CSIStorageCapacity, CSIStorageCapacityList, StorageClass, StorageClassList, TokenRequest, VolumeAttachment, VolumeAttachmentList, VolumeAttachmentSource, VolumeAttachmentSpec, VolumeAttachmentStatus, VolumeAttributesClass, VolumeAttributesClassList, VolumeError, VolumeNodeResources




__all__ = ['CSIDriver', 'CSIDriverList', 'CSIDriverSpec', 'CSINode', 'CSINodeDriver', 'CSINodeList', 'CSINodeSpec', 'CSIStorageCapacity', 'CSIStorageCapacityList', 'StorageClass', 'StorageClassList', 'TokenRequest', 'VolumeAttachment', 'VolumeAttachmentList', 'VolumeAttachmentSource', 'VolumeAttachmentSpec', 'VolumeAttachmentStatus', 'VolumeAttributesClass', 'VolumeAttributesClassList', 'VolumeError', 'VolumeNodeResources']




from __future__ import annotations

from typing import Type

from .loader import Loadable
from .registry import get_model, get_model_by_body
from .resource import K8sResource, K8sResourceList


def get_k8s_resource_model(api_version: str, kind: str) -> Type[K8sResource] | None:
    model = get_model(api_version, kind)
    if model is not None and issubclass(model, K8sResource):
        return model
    return None


__all__ = [
    "get_k8s_resource_model",
    "get_model",
    "get_model_by_body",
    "K8sResource",
    "K8sResourceList",
    "Loadable",
]



# generated by https://github.com/puzl-cloud/kubesdk
from .v1 import AggregationRule, ClusterRole, ClusterRoleBinding, ClusterRoleBindingList, ClusterRoleList, PolicyRule, Role, RoleBinding, RoleBindingList, RoleList, RoleRef, Subject




__all__ = ['AggregationRule', 'ClusterRole', 'ClusterRoleBinding', 'ClusterRoleBindingList', 'ClusterRoleList', 'PolicyRule', 'Role', 'RoleBinding', 'RoleBindingList', 'RoleList', 'RoleRef', 'Subject']




# generated by https://github.com/puzl-cloud/kubesdk
#   filename:  apis__policy__v1beta1_openapi.json

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime, timezone
from typing import ClassVar, Dict, List, Set

from kube_models.const import *
from kube_models.loader import *
from kube_models.loader import Loadable
from kube_models.resource import *

from ...apimachinery.pkg.apis.meta.v1 import (
    Condition,
    LabelSelector,
    ListMeta,
    ObjectMeta,
    Time,
)
from ...apimachinery.pkg.util.intstr import IntOrString
from ..core.v1 import SELinuxOptions


@dataclass(slots=True, kw_only=True, frozen=True)
class AllowedCSIDriver(Loadable):
    name: str


@dataclass(slots=True, kw_only=True, frozen=True)
class AllowedFlexVolume(Loadable):
    driver: str


@dataclass(slots=True, kw_only=True, frozen=True)
class AllowedHostPath(Loadable):
    pathPrefix: str | None = None
    readOnly: bool | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class HostPortRange(Loadable):
    max: int
    min: int


@dataclass(slots=True, kw_only=True, frozen=True)
class IDRange(Loadable):
    max: int
    min: int


@dataclass(slots=True, kw_only=True, frozen=True)
class RunAsGroupStrategyOptions(Loadable):
    rule: str
    ranges: List[IDRange] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class RunAsUserStrategyOptions(Loadable):
    rule: str
    ranges: List[IDRange] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class RuntimeClassStrategyOptions(Loadable):
    allowedRuntimeClassNames: List[str]
    defaultRuntimeClassName: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class SELinuxStrategyOptions(Loadable):
    rule: str
    seLinuxOptions: SELinuxOptions | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class SupplementalGroupsStrategyOptions(Loadable):
    ranges: List[IDRange] | None = None
    rule: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class FSGroupStrategyOptions(Loadable):
    ranges: List[IDRange] | None = None
    rule: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PodSecurityPolicySpec(Loadable):
    fsGroup: FSGroupStrategyOptions
    runAsUser: RunAsUserStrategyOptions
    seLinux: SELinuxStrategyOptions
    supplementalGroups: SupplementalGroupsStrategyOptions
    allowPrivilegeEscalation: bool | None = None
    allowedCSIDrivers: List[AllowedCSIDriver] | None = None
    allowedCapabilities: List[str] | None = None
    allowedFlexVolumes: List[AllowedFlexVolume] | None = None
    allowedHostPaths: List[AllowedHostPath] | None = None
    allowedProcMountTypes: List[str] | None = None
    allowedUnsafeSysctls: List[str] | None = None
    defaultAddCapabilities: List[str] | None = None
    defaultAllowPrivilegeEscalation: bool | None = None
    forbiddenSysctls: List[str] | None = None
    hostIPC: bool | None = None
    hostNetwork: bool | None = None
    hostPID: bool | None = None
    hostPorts: List[HostPortRange] | None = None
    privileged: bool | None = None
    readOnlyRootFilesystem: bool | None = None
    requiredDropCapabilities: List[str] | None = None
    runAsGroup: RunAsGroupStrategyOptions | None = None
    runtimeClass: RuntimeClassStrategyOptions | None = None
    volumes: List[str] | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PodDisruptionBudgetSpec(Loadable):
    maxUnavailable: IntOrString | None = None
    minAvailable: IntOrString | None = None
    selector: LabelSelector | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PodDisruptionBudgetStatus(Loadable):
    currentHealthy: int
    desiredHealthy: int
    disruptionsAllowed: int
    expectedPods: int
    conditions: List[Condition] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'type',
        },
    )
    disruptedPods: Dict[str, Time] | None = None
    observedGeneration: int | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class PodSecurityPolicy(K8sResource):
    apiVersion: ClassVar[str] = 'policy/v1beta1'
    kind: ClassVar[str] = 'PodSecurityPolicy'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    spec: PodSecurityPolicySpec | None = None
    plural_: ClassVar[str] = 'podsecuritypolicies'
    is_namespaced_: ClassVar[bool] = False
    group_: ClassVar[Optional[str]] = 'policy'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class PodSecurityPolicyList(Loadable):
    items: List[PodSecurityPolicy]
    apiVersion: str = 'policy/v1beta1'
    kind: str = 'PodSecurityPolicyList'
    metadata: ListMeta = field(default_factory=ObjectMeta)


@dataclass(slots=True, kw_only=True, frozen=True)
class PodDisruptionBudget(K8sResource):
    apiVersion: ClassVar[str] = 'policy/v1beta1'
    kind: ClassVar[str] = 'PodDisruptionBudget'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    spec: PodDisruptionBudgetSpec | None = None
    status: PodDisruptionBudgetStatus | None = None
    plural_: ClassVar[str] = 'poddisruptionbudgets'
    is_namespaced_: ClassVar[bool] = True
    group_: ClassVar[Optional[str]] = 'policy'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class PodDisruptionBudgetList(Loadable):
    items: List[PodDisruptionBudget]
    apiVersion: str = 'policy/v1beta1'
    kind: str = 'PodDisruptionBudgetList'
    metadata: ListMeta = field(default_factory=ObjectMeta)



# generated by https://github.com/puzl-cloud/kubesdk
from .v1 import ControllerRevision, ControllerRevisionList, DaemonSet, DaemonSetCondition, DaemonSetList, DaemonSetSpec, DaemonSetStatus, DaemonSetUpdateStrategy, Deployment, DeploymentCondition, DeploymentList, DeploymentSpec, DeploymentStatus, DeploymentStrategy, ReplicaSet, ReplicaSetCondition, ReplicaSetList, ReplicaSetSpec, ReplicaSetStatus, RollingUpdateDaemonSet, RollingUpdateDeployment, RollingUpdateStatefulSetStrategy, StatefulSet, StatefulSetCondition, StatefulSetList, StatefulSetOrdinals, StatefulSetPersistentVolumeClaimRetentionPolicy, StatefulSetSpec, StatefulSetStatus, StatefulSetUpdateStrategy




__all__ = ['ControllerRevision', 'ControllerRevisionList', 'DaemonSet', 'DaemonSetCondition', 'DaemonSetList', 'DaemonSetSpec', 'DaemonSetStatus', 'DaemonSetUpdateStrategy', 'Deployment', 'DeploymentCondition', 'DeploymentList', 'DeploymentSpec', 'DeploymentStatus', 'DeploymentStrategy', 'ReplicaSet', 'ReplicaSetCondition', 'ReplicaSetList', 'ReplicaSetSpec', 'ReplicaSetStatus', 'RollingUpdateDaemonSet', 'RollingUpdateDeployment', 'RollingUpdateStatefulSetStrategy', 'StatefulSet', 'StatefulSetCondition', 'StatefulSetList', 'StatefulSetOrdinals', 'StatefulSetPersistentVolumeClaimRetentionPolicy', 'StatefulSetSpec', 'StatefulSetStatus', 'StatefulSetUpdateStrategy']




# generated by https://github.com/puzl-cloud/kubesdk
#   filename:  apis__events.k8s.io__v1_openapi.json

from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime, timezone
from typing import ClassVar, Set

from kube_models.const import *
from kube_models.loader import *
from kube_models.loader import Loadable
from kube_models.resource import *


@dataclass(slots=True, kw_only=True, frozen=True)
class EventSource(Loadable):
    component: str | None = None
    host: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class ObjectReference(Loadable):
    apiVersion: str | None = None
    fieldPath: str | None = None
    kind: str | None = None
    name: str | None = None
    namespace: str | None = None
    resourceVersion: str | None = None
    uid: str | None = None



# generated by https://github.com/puzl-cloud/kubesdk
from .v1 import AllocatedDeviceStatus, AllocationResult, CELDeviceSelector, CapacityRequestPolicy, CapacityRequestPolicyRange, CapacityRequirements, Counter, CounterSet, Device, DeviceAllocationConfiguration, DeviceAllocationResult, DeviceAttribute, DeviceCapacity, DeviceClaim, DeviceClaimConfiguration, DeviceClass, DeviceClassConfiguration, DeviceClassList, DeviceClassSpec, DeviceConstraint, DeviceCounterConsumption, DeviceRequest, DeviceRequestAllocationResult, DeviceSelector, DeviceSubRequest, DeviceTaint, DeviceToleration, ExactDeviceRequest, NetworkDeviceData, OpaqueDeviceConfiguration, ResourceClaim, ResourceClaimConsumerReference, ResourceClaimList, ResourceClaimSpec, ResourceClaimStatus, ResourceClaimTemplate, ResourceClaimTemplateList, ResourceClaimTemplateSpec, ResourcePool, ResourceSlice, ResourceSliceList, ResourceSliceSpec




__all__ = ['AllocatedDeviceStatus', 'AllocationResult', 'CELDeviceSelector', 'CapacityRequestPolicy', 'CapacityRequestPolicyRange', 'CapacityRequirements', 'Counter', 'CounterSet', 'Device', 'DeviceAllocationConfiguration', 'DeviceAllocationResult', 'DeviceAttribute', 'DeviceCapacity', 'DeviceClaim', 'DeviceClaimConfiguration', 'DeviceClass', 'DeviceClassConfiguration', 'DeviceClassList', 'DeviceClassSpec', 'DeviceConstraint', 'DeviceCounterConsumption', 'DeviceRequest', 'DeviceRequestAllocationResult', 'DeviceSelector', 'DeviceSubRequest', 'DeviceTaint', 'DeviceToleration', 'ExactDeviceRequest', 'NetworkDeviceData', 'OpaqueDeviceConfiguration', 'ResourceClaim', 'ResourceClaimConsumerReference', 'ResourceClaimList', 'ResourceClaimSpec', 'ResourceClaimStatus', 'ResourceClaimTemplate', 'ResourceClaimTemplateList', 'ResourceClaimTemplateSpec', 'ResourcePool', 'ResourceSlice', 'ResourceSliceList', 'ResourceSliceSpec']




# generated by https://github.com/puzl-cloud/kubesdk
from .v1 import JobSpec
from .v1beta1 import CronJob, CronJobList, CronJobSpec, CronJobStatus, JobTemplateSpec




__all__ = ['JobSpec', 'CronJob', 'CronJobList', 'CronJobSpec', 'CronJobStatus', 'JobTemplateSpec']




# generated by https://github.com/puzl-cloud/kubesdk
#   filename:  apis__coordination.k8s.io__v1beta1_openapi.json

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime, timezone
from typing import ClassVar, List, Set

from kube_models.const import *
from kube_models.loader import *
from kube_models.loader import Loadable
from kube_models.resource import *

from ...apimachinery.pkg.apis.meta.v1 import ListMeta, MicroTime, ObjectMeta


@dataclass(slots=True, kw_only=True, frozen=True)
class LeaseCandidateSpec(Loadable):
    binaryVersion: str
    leaseName: str
    strategy: str
    emulationVersion: str | None = None
    pingTime: MicroTime | None = None
    renewTime: MicroTime | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class LeaseCandidate(K8sResource):
    apiVersion: ClassVar[str] = 'coordination.k8s.io/v1beta1'
    kind: ClassVar[str] = 'LeaseCandidate'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    spec: LeaseCandidateSpec | None = None
    plural_: ClassVar[str] = 'leasecandidates'
    is_namespaced_: ClassVar[bool] = True
    group_: ClassVar[Optional[str]] = 'coordination.k8s.io'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class LeaseCandidateList(Loadable):
    items: List[LeaseCandidate]
    apiVersion: str = 'coordination.k8s.io/v1beta1'
    kind: str = 'LeaseCandidateList'
    metadata: ListMeta = field(default_factory=ObjectMeta)



# generated by https://github.com/puzl-cloud/kubesdk
from .v1 import APIGroup, APIGroupList, GroupVersionForDiscovery, ServerAddressByClientCIDR




__all__ = ['APIGroup', 'APIGroupList', 'GroupVersionForDiscovery', 'ServerAddressByClientCIDR']




# generated by https://github.com/puzl-cloud/kubesdk
#   filename:  apis__apiregistration.k8s.io__v1_openapi.json

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime, timezone
from typing import ClassVar, List, Set

from kube_models.const import *
from kube_models.loader import *
from kube_models.loader import Loadable
from kube_models.resource import *

from .....apimachinery.pkg.apis.meta.v1 import ListMeta, ObjectMeta, Time


@dataclass(slots=True, kw_only=True, frozen=True)
class APIServiceCondition(Loadable):
    status: str
    type: str
    lastTransitionTime: Time | None = None
    message: str | None = None
    reason: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class APIServiceStatus(Loadable):
    conditions: List[APIServiceCondition] = field(
        default_factory=list,
        metadata={
            'x-kubernetes-patch-strategy': 'merge',
            'x-kubernetes-patch-merge-key': 'type',
        },
    )


@dataclass(slots=True, kw_only=True, frozen=True)
class ServiceReference(Loadable):
    name: str | None = None
    namespace: str | None = None
    port: int | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class APIServiceSpec(Loadable):
    groupPriorityMinimum: int
    versionPriority: int
    caBundle: str | None = None
    group: str | None = None
    insecureSkipTLSVerify: bool | None = None
    service: ServiceReference | None = None
    version: str | None = None


@dataclass(slots=True, kw_only=True, frozen=True)
class APIService(K8sResource):
    apiVersion: ClassVar[str] = 'apiregistration.k8s.io/v1'
    kind: ClassVar[str] = 'APIService'
    metadata: ObjectMeta = field(default_factory=ObjectMeta)
    spec: APIServiceSpec | None = None
    status: APIServiceStatus | None = None
    plural_: ClassVar[str] = 'apiservices'
    is_namespaced_: ClassVar[bool] = False
    group_: ClassVar[Optional[str]] = 'apiregistration.k8s.io'
    patch_strategies_: ClassVar[set[PatchRequestType]] = {
        'application/apply-patch+cbor',
        'application/apply-patch+yaml',
        'application/json-patch+json',
        'application/merge-patch+json',
        'application/strategic-merge-patch+json',
    }


@dataclass(slots=True, kw_only=True, frozen=True)
class APIServiceList(Loadable):
    items: List[APIService]
    apiVersion: str = 'apiregistration.k8s.io/v1'
    kind: str = 'APIServiceList'
    metadata: ListMeta = field(default_factory=ObjectMeta)



# generated by https://github.com/puzl-cloud/kubesdk
from .v1beta1 import AllowedCSIDriver, AllowedFlexVolume, AllowedHostPath, FSGroupStrategyOptions, HostPortRange, IDRange, PodDisruptionBudget, PodDisruptionBudgetList, PodDisruptionBudgetSpec, PodDisruptionBudgetStatus, PodSecurityPolicy, PodSecurityPolicyList, PodSecurityPolicySpec, RunAsGroupStrategyOptions, RunAsUserStrategyOptions, RuntimeClassStrategyOptions, SELinuxStrategyOptions, SupplementalGroupsStrategyOptions




__all__ = ['AllowedCSIDriver', 'AllowedFlexVolume', 'AllowedHostPath', 'FSGroupStrategyOptions', 'HostPortRange', 'IDRange', 'PodDisruptionBudget', 'PodDisruptionBudgetList', 'PodDisruptionBudgetSpec', 'PodDisruptionBudgetStatus', 'PodSecurityPolicy', 'PodSecurityPolicyList', 'PodSecurityPolicySpec', 'RunAsGroupStrategyOptions', 'RunAsUserStrategyOptions', 'RuntimeClassStrategyOptions', 'SELinuxStrategyOptions', 'SupplementalGroupsStrategyOptions']




# generated by https://github.com/puzl-cloud/kubesdk
from .v1 import AWSElasticBlockStoreVolumeSource, Affinity, AppArmorProfile, AzureDiskVolumeSource, AzureFileVolumeSource, CSIVolumeSource, Capabilities, CephFSVolumeSource, CinderVolumeSource, ClusterTrustBundleProjection, ConfigMapEnvSource, ConfigMapKeySelector, ConfigMapProjection, ConfigMapVolumeSource, Container, ContainerPort, ContainerResizePolicy, ContainerRestartRule, ContainerRestartRuleOnExitCodes, DownwardAPIProjection, DownwardAPIVolumeFile, DownwardAPIVolumeSource, EmptyDirVolumeSource, EnvFromSource, EnvVar, EnvVarSource, EphemeralContainer, EphemeralVolumeSource, ExecAction, FCVolumeSource, FileKeySelector, FlexVolumeSource, FlockerVolumeSource, GCEPersistentDiskVolumeSource, GRPCAction, GitRepoVolumeSource, GlusterfsVolumeSource, HTTPGetAction, HTTPHeader, HostAlias, HostPathVolumeSource, ISCSIVolumeSource, ImageVolumeSource, KeyToPath, Lifecycle, LifecycleHandler, LocalObjectReference, NFSVolumeSource, NodeAffinity, NodeSelector, NodeSelectorRequirement, NodeSelectorTerm, ObjectFieldSelector, ObjectReference, PersistentVolumeClaimSpec, PersistentVolumeClaimTemplate, PersistentVolumeClaimVolumeSource, PhotonPersistentDiskVolumeSource, PodAffinity, PodAffinityTerm, PodAntiAffinity, PodCertificateProjection, PodDNSConfig, PodDNSConfigOption, PodOS, PodReadinessGate, PodResourceClaim, PodSchedulingGate, PodSecurityContext, PodSpec, PodTemplateSpec, PortworxVolumeSource, PreferredSchedulingTerm, Probe, ProjectedVolumeSource, QuobyteVolumeSource, RBDVolumeSource, ResourceClaim, ResourceFieldSelector, ResourceRequirements, SELinuxOptions, ScaleIOVolumeSource, SeccompProfile, SecretEnvSource, SecretKeySelector, SecretProjection, SecretVolumeSource, SecurityContext, ServiceAccountTokenProjection, SleepAction, StorageOSVolumeSource, Sysctl, TCPSocketAction, Toleration, TopologySpreadConstraint, TypedLocalObjectReference, TypedObjectReference, Volume, VolumeDevice, VolumeMount, VolumeProjection, VolumeResourceRequirements, VsphereVirtualDiskVolumeSource, WeightedPodAffinityTerm, WindowsSecurityContextOptions




__all__ = ['AWSElasticBlockStoreVolumeSource', 'Affinity', 'AppArmorProfile', 'AzureDiskVolumeSource', 'AzureFileVolumeSource', 'CSIVolumeSource', 'Capabilities', 'CephFSVolumeSource', 'CinderVolumeSource', 'ClusterTrustBundleProjection', 'ConfigMapEnvSource', 'ConfigMapKeySelector', 'ConfigMapProjection', 'ConfigMapVolumeSource', 'Container', 'ContainerPort', 'ContainerResizePolicy', 'ContainerRestartRule', 'ContainerRestartRuleOnExitCodes', 'DownwardAPIProjection', 'DownwardAPIVolumeFile', 'DownwardAPIVolumeSource', 'EmptyDirVolumeSource', 'EnvFromSource', 'EnvVar', 'EnvVarSource', 'EphemeralContainer', 'EphemeralVolumeSource', 'ExecAction', 'FCVolumeSource', 'FileKeySelector', 'FlexVolumeSource', 'FlockerVolumeSource', 'GCEPersistentDiskVolumeSource', 'GRPCAction', 'GitRepoVolumeSource', 'GlusterfsVolumeSource', 'HTTPGetAction', 'HTTPHeader', 'HostAlias', 'HostPathVolumeSource', 'ISCSIVolumeSource', 'ImageVolumeSource', 'KeyToPath', 'Lifecycle', 'LifecycleHandler', 'LocalObjectReference', 'NFSVolumeSource', 'NodeAffinity', 'NodeSelector', 'NodeSelectorRequirement', 'NodeSelectorTerm', 'ObjectFieldSelector', 'ObjectReference', 'PersistentVolumeClaimSpec', 'PersistentVolumeClaimTemplate', 'PersistentVolumeClaimVolumeSource', 'PhotonPersistentDiskVolumeSource', 'PodAffinity', 'PodAffinityTerm', 'PodAntiAffinity', 'PodCertificateProjection', 'PodDNSConfig', 'PodDNSConfigOption', 'PodOS', 'PodReadinessGate', 'PodResourceClaim', 'PodSchedulingGate', 'PodSecurityContext', 'PodSpec', 'PodTemplateSpec', 'PortworxVolumeSource', 'PreferredSchedulingTerm', 'Probe', 'ProjectedVolumeSource', 'QuobyteVolumeSource', 'RBDVolumeSource', 'ResourceClaim', 'ResourceFieldSelector', 'ResourceRequirements', 'SELinuxOptions', 'ScaleIOVolumeSource', 'SeccompProfile', 'SecretEnvSource', 'SecretKeySelector', 'SecretProjection', 'SecretVolumeSource', 'SecurityContext', 'ServiceAccountTokenProjection', 'SleepAction', 'StorageOSVolumeSource', 'Sysctl', 'TCPSocketAction', 'Toleration', 'TopologySpreadConstraint', 'TypedLocalObjectReference', 'TypedObjectReference', 'Volume', 'VolumeDevice', 'VolumeMount', 'VolumeProjection', 'VolumeResourceRequirements', 'VsphereVirtualDiskVolumeSource', 'WeightedPodAffinityTerm', 'WindowsSecurityContextOptions']




# generated by https://github.com/puzl-cloud/kubesdk
#   filename:  apis__resource.k8s.io__v1beta1_openapi.json

from __future__ import annotations

from datetime import datetime, timezone
from typing import ClassVar, Set

from kube_models.const import *
from kube_models.loader import *
from kube_models.loader import Loadable
from kube_models.resource import *

Quantity = str | float
